@@ -50,7 +50,6 @@ namespace Pinta.Core
 		public Gtk.Action SavePalette { get; private set; }
 		public Gtk.Action ResetPalette { get; private set; }
 		public Gtk.Action ResizePalette { get; private set; }
-		public Gtk.Action AddinManager { get; private set; }
 		
 		private string lastPaletteDir = null;
 		
@@ -84,8 +83,6 @@ namespace Pinta.Core
 			ResetPalette = new Gtk.Action ("ResetPalette", Catalog.GetString ("Reset to Default"), null, Stock.RevertToSaved);
 			ResizePalette = new Gtk.Action ("ResizePalette", Catalog.GetString ("Set Number of Colors"), null, "Menu.Image.Resize.png");
 
-			AddinManager = new Gtk.Action ("AddinManager", Catalog.GetString ("Add-in Manager"), null, "Menu.Edit.Addins.png");
-			
 			Undo.IsImportant = true;
 			Undo.Sensitive = false;
 			Redo.Sensitive = false;
@@ -122,7 +119,7 @@ namespace Pinta.Core
 			menu.AppendSeparator ();
 			menu.Append (EraseSelection.CreateAcceleratedMenuItem (Gdk.Key.Delete, Gdk.ModifierType.None));
 			menu.Append (FillSelection.CreateAcceleratedMenuItem (Gdk.Key.BackSpace, Gdk.ModifierType.None));
-			//menu.Append (InvertSelection.CreateAcceleratedMenuItem (Gdk.Key.I, Gdk.ModifierType.ControlMask));
+			menu.Append (InvertSelection.CreateAcceleratedMenuItem (Gdk.Key.I, Gdk.ModifierType.ControlMask));
 			
 			menu.AppendSeparator ();
 			Gtk.Action menu_action = new Gtk.Action ("Palette", Mono.Unix.Catalog.GetString ("Palette"), null, null);
@@ -131,9 +128,6 @@ namespace Pinta.Core
 			palette_menu.Append (SavePalette.CreateMenuItem ());
 			palette_menu.Append (ResetPalette.CreateMenuItem ());
 			palette_menu.Append (ResizePalette.CreateMenuItem ());
-
-			menu.AppendSeparator ();
-			menu.Append (AddinManager.CreateMenuItem ());
 		}
 
 		public void CreateHistoryWindowToolBar (Gtk.Toolbar toolbar)
@@ -156,9 +150,11 @@ namespace Pinta.Core
 			LoadPalette.Activated += HandlerPintaCoreActionsEditLoadPaletteActivated;
 			SavePalette.Activated += HandlerPintaCoreActionsEditSavePaletteActivated;
 			ResetPalette.Activated += HandlerPintaCoreActionsEditResetPaletteActivated;
+			InvertSelection.Activated += HandleInvertSelectionActivated;
 
 			PintaCore.Workspace.ActiveDocumentChanged += WorkspaceActiveDocumentChanged;
 		}
+
 		#endregion
 
 		#region Action Handlers
@@ -168,18 +164,18 @@ namespace Pinta.Core
 
 			PintaCore.Tools.Commit ();
 
-			Cairo.ImageSurface old = doc.CurrentLayer.Surface.Clone ();
+			Cairo.ImageSurface old = doc.CurrentUserLayer.Surface.Clone ();
 
-			using (var g = new Cairo.Context (doc.CurrentLayer.Surface)) {
-				g.AppendPath(doc.Selection.SelectionPath);
+			using (var g = new Cairo.Context (doc.CurrentUserLayer.Surface)) {
+				g.AppendPath (doc.Selection.SelectionPath);
 				g.FillRule = FillRule.EvenOdd;
 
-				g.Color = PintaCore.Palette.PrimaryColor;
+				g.SetSourceColor (PintaCore.Palette.PrimaryColor);
 				g.Fill ();
 			}
 
 			doc.Workspace.Invalidate ();
-			doc.History.PushNewItem (new SimpleHistoryItem ("Menu.Edit.FillSelection.png", Catalog.GetString ("Fill Selection"), old, doc.CurrentLayerIndex));
+			doc.History.PushNewItem (new SimpleHistoryItem ("Menu.Edit.FillSelection.png", Catalog.GetString ("Fill Selection"), old, doc.CurrentUserLayerIndex));
 		}
 
 		private void HandlePintaCoreActionsEditSelectAllActivated (object sender, EventArgs e)
@@ -191,7 +187,7 @@ namespace Pinta.Core
 			SelectionHistoryItem hist = new SelectionHistoryItem (Stock.SelectAll, Catalog.GetString ("Select All"));
 			hist.TakeSnapshot ();
 
-			doc.ResetSelectionPath ();
+			doc.ResetSelectionPaths ();
 			doc.ShowSelection = true;
 
 			doc.History.PushNewItem (hist);
@@ -204,10 +200,10 @@ namespace Pinta.Core
 
 			PintaCore.Tools.Commit ();
 
-			Cairo.ImageSurface old = doc.CurrentLayer.Surface.Clone ();
+			Cairo.ImageSurface old = doc.CurrentUserLayer.Surface.Clone ();
 
-			using (var g = new Cairo.Context (doc.CurrentLayer.Surface)) {
-				g.AppendPath(doc.Selection.SelectionPath);
+			using (var g = new Cairo.Context (doc.CurrentUserLayer.Surface)) {
+				g.AppendPath (doc.Selection.SelectionPath);
 				g.FillRule = FillRule.EvenOdd;
 
 				g.Operator = Cairo.Operator.Clear;
@@ -217,9 +213,9 @@ namespace Pinta.Core
 			doc.Workspace.Invalidate ();
 
 			if (sender is string && (sender as string) == "Cut")
-				doc.History.PushNewItem (new SimpleHistoryItem (Stock.Cut, Catalog.GetString ("Cut"), old, doc.CurrentLayerIndex));
+				doc.History.PushNewItem (new SimpleHistoryItem (Stock.Cut, Catalog.GetString ("Cut"), old, doc.CurrentUserLayerIndex));
 			else
-				doc.History.PushNewItem (new SimpleHistoryItem ("Menu.Edit.EraseSelection.png", Catalog.GetString ("Erase Selection"), old, doc.CurrentLayerIndex));
+				doc.History.PushNewItem (new SimpleHistoryItem ("Menu.Edit.EraseSelection.png", Catalog.GetString ("Erase Selection"), old, doc.CurrentUserLayerIndex));
 		}
 
 		private void HandlePintaCoreActionsEditDeselectActivated (object sender, EventArgs e)
@@ -231,7 +227,11 @@ namespace Pinta.Core
 			SelectionHistoryItem hist = new SelectionHistoryItem ("Menu.Edit.Deselect.png", Catalog.GetString ("Deselect"));
 			hist.TakeSnapshot ();
 
-			doc.ResetSelectionPath ();
+			doc.ResetSelectionPaths ();
+			doc.Selection.SelectionPolygons.Clear();
+            doc.PreviousSelection.SelectionPolygons.Clear ();
+			doc.Selection.selOrigin = new PointD();
+			doc.Selection.selEnd = new PointD();
 
 			doc.History.PushNewItem (hist);
 			doc.Workspace.Invalidate ();
@@ -247,21 +247,23 @@ namespace Pinta.Core
 
 			PintaCore.Tools.Commit ();
 
-			ImageSurface src = doc.GetClippedLayer (doc.CurrentLayerIndex);
+			using (ImageSurface src = doc.GetClippedLayer (doc.CurrentUserLayerIndex)) {
 
-			Gdk.Rectangle rect = doc.GetSelectedBounds (true);
+				Gdk.Rectangle rect = doc.GetSelectedBounds (true);
+				if (rect.Width == 0 || rect.Height == 0)
+					return;
 			
-			ImageSurface dest = new ImageSurface (Format.Argb32, rect.Width, rect.Height);
+				ImageSurface dest = new ImageSurface (Format.Argb32, rect.Width, rect.Height);
 
-			using (Context g = new Context (dest)) {
-				g.SetSourceSurface (src, -rect.X, -rect.Y);
-				g.Paint ();
+				using (Context g = new Context (dest)) {
+					g.SetSourceSurface (src, -rect.X, -rect.Y);
+					g.Paint ();
+				}
+			
+				cb.Image = dest.ToPixbuf ();
+
+				(dest as IDisposable).Dispose ();
 			}
-			
-			cb.Image = dest.ToPixbuf ();
-
-			(src as IDisposable).Dispose ();
-			(dest as IDisposable).Dispose ();
 		}
 
 		private void HandlerPintaCoreActionsEditCopyMergedActivated (object sender, EventArgs e)
@@ -293,7 +295,6 @@ namespace Pinta.Core
 			Gtk.Clipboard cb = Gtk.Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
 			if (PintaCore.Tools.CurrentTool.TryHandleCut (cb))
 				return;
-			Document doc = PintaCore.Workspace.ActiveDocument;
 			PintaCore.Tools.Commit ();
 			
 			// Copy selection
@@ -309,6 +310,7 @@ namespace Pinta.Core
 				return;
 			Document doc = PintaCore.Workspace.ActiveDocument;
 			doc.History.Undo ();
+			PintaCore.Tools.CurrentTool.AfterUndo();
 		}
 
 		private void HandlerPintaCoreActionsEditRedoActivated (object sender, EventArgs e)
@@ -317,6 +319,7 @@ namespace Pinta.Core
 				return;
 			Document doc = PintaCore.Workspace.ActiveDocument;
 			doc.History.Redo ();
+			PintaCore.Tools.CurrentTool.AfterRedo();
 		}
 
 		private void HandlerPintaCoreActionsEditLoadPaletteActivated (object sender, EventArgs e)
@@ -384,7 +387,14 @@ namespace Pinta.Core
 		
 			if (response == (int) Gtk.ResponseType.Ok) {
 				Palette.FileFormat format = (fcd.Filter == ffPDN) ? Palette.FileFormat.PDN : Palette.FileFormat.GIMP;
-				PintaCore.Palette.CurrentPalette.Save (fcd.Filename, format);
+
+				string fileExtension = (fcd.Filter == ffPDN) ? ".txt" : ".gpl";
+				string finalFileName = fcd.Filename;
+				bool fileNameHasExtension = finalFileName.EndsWith(".txt") || finalFileName.EndsWith(".gpl");
+				if (!fileNameHasExtension)
+					finalFileName += fileExtension;
+
+				PintaCore.Palette.CurrentPalette.Save (finalFileName, format);
 			}
 
 			fcd.Destroy ();
@@ -395,6 +405,25 @@ namespace Pinta.Core
 			PintaCore.Palette.CurrentPalette.LoadDefault ();
 		}
 
+		void HandleInvertSelectionActivated (object sender, EventArgs e)
+		{
+			PintaCore.Tools.Commit ();
+
+			Document doc = PintaCore.Workspace.ActiveDocument;
+
+			// Clear the selection resize handles if necessary.
+			doc.ToolLayer.Clear ();
+
+			SelectionHistoryItem historyItem = new SelectionHistoryItem ("Menu.Edit.InvertSelection.png",
+			                                                             Catalog.GetString ("Invert Selection"));
+			historyItem.TakeSnapshot ();
+
+			doc.Selection.Invert (doc.SelectionLayer.Surface, doc.ImageSize);
+
+			doc.History.PushNewItem (historyItem);
+			doc.Workspace.Invalidate ();
+		}
+
 		private void WorkspaceActiveDocumentChanged (object sender, EventArgs e)
 		{
 			if (!PintaCore.Workspace.HasOpenDocuments) {
