@@ -109,7 +109,7 @@ namespace Pinta.Core
 			PintaCore.Tools.Commit ();
 			doc.RotateImageCCW ();
 
-			doc.ResetSelectionPath ();
+			doc.ResetSelectionPaths ();
 
 			doc.History.PushNewItem (new InvertHistoryItem (InvertType.Rotate90CCW));
 		}
@@ -121,7 +121,7 @@ namespace Pinta.Core
 			PintaCore.Tools.Commit ();
 			doc.RotateImageCW ();
 
-			doc.ResetSelectionPath ();
+			doc.ResetSelectionPaths ();
 
 			doc.History.PushNewItem (new InvertHistoryItem (InvertType.Rotate90CW));
 		}
@@ -132,12 +132,12 @@ namespace Pinta.Core
 
 			PintaCore.Tools.Commit ();
 
-			var oldBottomSurface = doc.Layers[0].Surface.Clone ();
+			var oldBottomSurface = doc.UserLayers[0].Surface.Clone ();
 
 			CompoundHistoryItem hist = new CompoundHistoryItem ("Menu.Image.Flatten.png", Catalog.GetString ("Flatten"));
 
-			for (int i = doc.Layers.Count - 1; i >= 1; i--)
-				hist.Push (new DeleteLayerHistoryItem (string.Empty, string.Empty, doc.Layers[i], i));
+			for (int i = doc.UserLayers.Count - 1; i >= 1; i--)
+				hist.Push (new DeleteLayerHistoryItem (string.Empty, string.Empty, doc.UserLayers[i], i));
 
 			doc.FlattenImage ();
 
@@ -152,7 +152,7 @@ namespace Pinta.Core
 			PintaCore.Tools.Commit ();
 			doc.RotateImage180 ();
 
-			doc.ResetSelectionPath ();
+			doc.ResetSelectionPaths ();
 
 			doc.History.PushNewItem (new InvertHistoryItem (InvertType.Rotate180));
 		}
@@ -194,7 +194,7 @@ namespace Pinta.Core
 
 			PintaCore.Tools.Commit ();
 
-			Cairo.ImageSurface image = doc.CurrentLayer.Surface;
+			Cairo.ImageSurface image = doc.CurrentUserLayer.Surface;
 			Gdk.Rectangle rect = image.GetBounds ();
 
 			Cairo.Color borderColor = image.GetPixel (0, 0);
@@ -271,33 +271,36 @@ namespace Pinta.Core
 
 		static void CropImageToRectangle (Document doc, Gdk.Rectangle rect)
 		{
-			ResizeHistoryItem hist = new ResizeHistoryItem (doc.ImageSize);
-			
-			hist.Icon = "Menu.Image.Crop.png";
-			hist.Text = Catalog.GetString ("Crop to Selection");
-			hist.StartSnapshotOfImage ();
-			hist.RestoreSelection = doc.Selection.Clone();
-			
-			PintaCore.Chrome.Canvas.GdkWindow.FreezeUpdates ();
-			
-			double original_scale = doc.Workspace.Scale;
-			doc.ImageSize = rect.Size;
-			doc.Workspace.CanvasSize = rect.Size;
-			doc.Workspace.Scale = original_scale;
-			
-			PintaCore.Actions.View.UpdateCanvasScale ();
-			
-			PintaCore.Chrome.Canvas.GdkWindow.ThawUpdates ();
-			
-			foreach (var layer in doc.Layers)
-				layer.Crop (rect, doc.Selection.SelectionPath);
+			if (rect.Width > 0 && rect.Height > 0)
+			{
+				ResizeHistoryItem hist = new ResizeHistoryItem(doc.ImageSize);
 
-			hist.FinishSnapshotOfImage ();
-			
-			doc.History.PushNewItem (hist);
-			doc.ResetSelectionPath ();
-			
-			doc.Workspace.Invalidate ();
+				hist.Icon = "Menu.Image.Crop.png";
+				hist.Text = Catalog.GetString("Crop to Selection");
+				hist.StartSnapshotOfImage();
+				hist.RestoreSelection = doc.Selection.Clone();
+
+				PintaCore.Chrome.Canvas.GdkWindow.FreezeUpdates();
+
+				double original_scale = doc.Workspace.Scale;
+				doc.ImageSize = rect.Size;
+				doc.Workspace.CanvasSize = rect.Size;
+				doc.Workspace.Scale = original_scale;
+
+				PintaCore.Actions.View.UpdateCanvasScale();
+
+				PintaCore.Chrome.Canvas.GdkWindow.ThawUpdates();
+
+				foreach (var layer in doc.UserLayers)
+					layer.Crop(rect);
+
+				hist.FinishSnapshotOfImage();
+
+				doc.History.PushNewItem(hist);
+				doc.ResetSelectionPaths();
+
+				doc.Workspace.Invalidate();
+			}
 		}
 	}
 }
