@@ -1,4 +1,4 @@
-﻿// 
+// 
 // Document.cs
 //  
 // Author:
@@ -28,9 +28,11 @@ using System;
 using System.Linq;
 using Mono.Unix;
 using Gdk;
+using Gtk;
 using System.Collections.Generic;
 using Cairo;
 using System.ComponentModel;
+using Pinta;
 
 namespace Pinta.Core
 {
@@ -47,13 +49,14 @@ namespace Pinta.Core
 
 		// The layer for tools to use until their output is committed
 		private Layer tool_layer;
+
 		// The layer used for selections
 		private Layer selection_layer;
 
-		private int selection_layer_index;
 		private bool show_selection;
 
-		public DocumentSelection Selection = new DocumentSelection();
+        public DocumentSelection Selection = new DocumentSelection ();
+        public DocumentSelection PreviousSelection = new DocumentSelection ();
 
 		public Document (Gdk.Size size)
 		{
@@ -62,9 +65,10 @@ namespace Pinta.Core
 			Workspace = new DocumentWorkspace (this);
 			IsDirty = false;
 			HasFile = false;
+			HasBeenSavedInSession = false;
 			ImageSize = size;
-			
-			Layers = new List<Layer> ();
+
+			UserLayers = new List<UserLayer>();
 
 			tool_layer = CreateLayer ("Tool Layer");
 			tool_layer.Hidden = true;
@@ -72,15 +76,16 @@ namespace Pinta.Core
 			selection_layer = CreateLayer ("Selection Layer");
 			selection_layer.Hidden = true;
 
-			ResetSelectionPath ();
+			ResetSelectionPaths ();
 		}
 
 		#region Public Properties
-		public Layer CurrentLayer {
-			get { return Layers[current_layer]; }
+		public UserLayer CurrentUserLayer
+		{
+			get { return UserLayers[current_layer]; }
 		}
 
-		public int CurrentLayerIndex {
+		public int CurrentUserLayerIndex {
 			get { return current_layer; }
 		}
 		
@@ -100,6 +105,11 @@ namespace Pinta.Core
 		public Guid Guid { get; private set; }
 		
 		public bool HasFile { get; set; }
+
+		//Determines whether or not the Document has been saved to the file that it is currently associated with in the
+		//current session. This should be false if the Document has not yet been saved, if it was just loaded into
+		//Pinta from a file, or if the user just clicked Save As.
+		public bool HasBeenSavedInSession { get; set; }
 		
 		public DocumentWorkspaceHistory History { get { return Workspace.History; } }
 
@@ -115,7 +125,7 @@ namespace Pinta.Core
 			}
 		}
 		
-		public List<Layer> Layers { get; private set; }
+		public List<UserLayer> UserLayers { get; private set; }
 
 		/// <summary>
 		/// Just the directory name, like "C:\MyPictures".
@@ -150,6 +160,7 @@ namespace Pinta.Core
 				PintaCore.Actions.Edit.EraseSelection.Sensitive = show_selection;
 				PintaCore.Actions.Edit.FillSelection.Sensitive = show_selection;
 				PintaCore.Actions.Image.CropToSelection.Sensitive = show_selection;
+				PintaCore.Actions.Edit.InvertSelection.Sensitive = show_selection;
 			}
 		}
 
@@ -168,22 +179,24 @@ namespace Pinta.Core
 		}
 
 		public DocumentWorkspace Workspace { get; private set; }
+
+		public delegate void LayerCloneEvent();
 		#endregion
 
 		#region Public Methods
 		// Adds a new layer above the current one
-		public Layer AddNewLayer (string name)
+		public UserLayer AddNewLayer(string name)
 		{
-			Layer layer;
+			UserLayer layer;
 
 			if (string.IsNullOrEmpty (name))
 				layer = CreateLayer ();
 			else
 				layer = CreateLayer (name);
 
-			Layers.Insert (current_layer + 1, layer);
+			UserLayers.Insert (current_layer + 1, layer);
 
-			if (Layers.Count == 1)
+			if (UserLayers.Count == 1)
 				current_layer = 0;
 
 			layer.PropertyChanged += RaiseLayerPropertyChangedEvent;
@@ -204,9 +217,9 @@ namespace Pinta.Core
 
 		public void Clear ()
 		{
-			while (Layers.Count > 0) {
-				Layer l = Layers[Layers.Count - 1];
-				Layers.RemoveAt (Layers.Count - 1);
+			while (UserLayers.Count > 0) {
+				Layer l = UserLayers[UserLayers.Count - 1];
+				UserLayers.RemoveAt (UserLayers.Count - 1);
 				(l.Surface as IDisposable).Dispose ();
 			}
 
@@ -218,9 +231,9 @@ namespace Pinta.Core
 		public void Close ()
 		{
 			// Dispose all of our layers
-			while (Layers.Count > 0) {
-				Layer l = Layers[Layers.Count - 1];
-				Layers.RemoveAt (Layers.Count - 1);
+			while (UserLayers.Count > 0) {
+				Layer l = UserLayers[UserLayers.Count - 1];
+				UserLayers.RemoveAt (UserLayers.Count - 1);
 				(l.Surface as IDisposable).Dispose ();
 			}
 
@@ -232,73 +245,61 @@ namespace Pinta.Core
 			if (selection_layer != null)
 				(selection_layer.Surface as IDisposable).Dispose ();
 
-			Selection.DisposeSelection();
+            Selection.Dispose ();
+            PreviousSelection.Dispose ();
 
-            Workspace.History.Clear ();
+			Workspace.History.Clear ();
 		}
 
 		public Context CreateClippedContext ()
 		{
-			Context g = new Context (CurrentLayer.Surface);
-
-			g.AppendPath (Selection.SelectionPath);
-			g.FillRule = Cairo.FillRule.EvenOdd;
-			g.Clip ();
-
+			Context g = new Context (CurrentUserLayer.Surface);
+			Selection.Clip (g);
 			return g;
 		}
 
 		public Context CreateClippedContext (bool antialias)
 		{
-			Context g = new Context (CurrentLayer.Surface);
-
-			g.AppendPath(Selection.SelectionPath);
-			g.FillRule = Cairo.FillRule.EvenOdd;
-			g.Clip ();
-
+			Context g = new Context (CurrentUserLayer.Surface);
+			Selection.Clip (g);
 			g.Antialias = antialias ? Antialias.Subpixel : Antialias.None;
-
 			return g;
 		}
 
 		public Context CreateClippedToolContext ()
 		{
 			Context g = new Context (ToolLayer.Surface);
-
-			g.AppendPath(Selection.SelectionPath);
-			g.FillRule = Cairo.FillRule.EvenOdd;
-			g.Clip ();
-
+			Selection.Clip (g);
 			return g;
 		}
 
 		public Context CreateClippedToolContext (bool antialias)
 		{
 			Context g = new Context (ToolLayer.Surface);
-
-			g.AppendPath(Selection.SelectionPath);
-			g.FillRule = Cairo.FillRule.EvenOdd;
-			g.Clip ();
-
+			Selection.Clip (g);
 			g.Antialias = antialias ? Antialias.Subpixel : Antialias.None;
-
 			return g;
 		}
 
-		public Layer CreateLayer ()
+		public UserLayer CreateLayer ()
 		{
 			return CreateLayer (string.Format ("{0} {1}", Catalog.GetString ("Layer"), layer_name_int++));
 		}
 
-		public Layer CreateLayer (string name)
+		public UserLayer CreateLayer (int width, int height)
+		{
+			return CreateLayer (string.Format ("{0} {1}", Catalog.GetString ("Layer"), layer_name_int++), width, height);
+		}
+
+		public UserLayer CreateLayer (string name)
 		{
 			return CreateLayer (name, ImageSize.Width, ImageSize.Height);
 		}
 
-		public Layer CreateLayer (string name, int width, int height)
+		public UserLayer CreateLayer(string name, int width, int height)
 		{
 			Cairo.ImageSurface surface = new Cairo.ImageSurface (Cairo.Format.ARGB32, width, height);
-			Layer layer = new Layer (surface) { Name = name };
+			UserLayer layer = new UserLayer(surface) { Name = name };
 
 			return layer;
 		}
@@ -308,7 +309,16 @@ namespace Pinta.Core
 			Layer old = selection_layer;
 
 			selection_layer = CreateLayer ();
-			selection_layer_index = current_layer + 1;
+
+			if (old != null)
+				(old.Surface as IDisposable).Dispose ();
+		}
+
+		public void CreateSelectionLayer (int width, int height)
+		{
+			Layer old = selection_layer;
+
+			selection_layer = CreateLayer (width, height);
 
 			if (old != null)
 				(old.Surface as IDisposable).Dispose ();
@@ -317,9 +327,9 @@ namespace Pinta.Core
 		// Delete the current layer
 		public void DeleteCurrentLayer ()
 		{
-			Layer layer = CurrentLayer;
+			Layer layer = CurrentUserLayer;
 
-			Layers.RemoveAt (current_layer);
+			UserLayers.RemoveAt (current_layer);
 
 			// Only change this if this wasn't already the bottom layer
 			if (current_layer > 0)
@@ -333,9 +343,9 @@ namespace Pinta.Core
 		// Delete the layer
 		public void DeleteLayer (int index, bool dispose)
 		{
-			Layer layer = Layers[index];
+			Layer layer = UserLayers[index];
 
-			Layers.RemoveAt (index);
+			UserLayers.RemoveAt (index);
 
 			if (dispose)
 				(layer.Surface as IDisposable).Dispose ();
@@ -353,14 +363,14 @@ namespace Pinta.Core
 		{
 			ShowSelectionLayer = false;
 			SelectionLayer.Clear ();
-			SelectionLayer.Offset = new PointD (0, 0);
+			SelectionLayer.Transform.InitIdentity();
 		}
 
 		// Duplicate current layer
-		public Layer DuplicateCurrentLayer ()
+		public UserLayer DuplicateCurrentLayer()
 		{
-			Layer source = CurrentLayer;
-			Layer layer = CreateLayer (string.Format ("{0} {1}", source.Name, Catalog.GetString ("copy")));
+			UserLayer source = CurrentUserLayer;
+			UserLayer layer = CreateLayer(string.Format("{0} {1}", source.Name, Catalog.GetString("copy")));
 
 			using (Cairo.Context g = new Cairo.Context (layer.Surface)) {
 				g.SetSource (source.Surface);
@@ -371,7 +381,7 @@ namespace Pinta.Core
 			layer.Opacity = source.Opacity;
 			layer.Tiled = source.Tiled;
 
-			Layers.Insert (++current_layer, layer);
+			UserLayers.Insert (++current_layer, layer);
 
 			layer.PropertyChanged += RaiseLayerPropertyChangedEvent;
 
@@ -391,13 +401,8 @@ namespace Pinta.Core
 
 			Layer layer = SelectionLayer;
 
-			using (Cairo.Context g = new Cairo.Context (CurrentLayer.Surface)) {
-				g.Save ();
-
-				g.SetSourceSurface (layer.Surface, (int)layer.Offset.X, (int)layer.Offset.Y);
-				g.PaintWithAlpha (layer.Opacity);
-
-				g.Restore ();
+			using (Cairo.Context g = new Cairo.Context (CurrentUserLayer.Surface)) {
+				layer.Draw(g);
 			}
 
 			DestroySelectionLayer ();
@@ -409,11 +414,11 @@ namespace Pinta.Core
 		// Flatten image
 		public void FlattenImage ()
 		{
-			if (Layers.Count < 2)
+			if (UserLayers.Count < 2)
 				throw new InvalidOperationException ("Cannot flatten image because there is only one layer.");
 
 			// Find the "bottom" layer
-			var bottom_layer = Layers[0];
+			var bottom_layer = UserLayers[0];
 			var old_surf = bottom_layer.Surface;
 
 			// Replace the bottom surface with the flattened image,
@@ -425,8 +430,8 @@ namespace Pinta.Core
 			current_layer = 0;
 
 			// Delete all other layers
-			while (Layers.Count > 1)
-				Layers.RemoveAt (1);
+			while (UserLayers.Count > 1)
+				UserLayers.RemoveAt (1);
 
 			PintaCore.Layers.OnLayerRemoved ();
 			Workspace.Invalidate ();
@@ -435,7 +440,7 @@ namespace Pinta.Core
 		// Flip image horizontally
 		public void FlipImageHorizontal ()
 		{
-			foreach (var layer in Layers)
+			foreach (var layer in UserLayers)
 				layer.FlipHorizontal ();
 
 			Workspace.Invalidate ();
@@ -444,7 +449,7 @@ namespace Pinta.Core
 		// Flip image vertically
 		public void FlipImageVertical ()
 		{
-			foreach (var layer in Layers)
+			foreach (var layer in UserLayers)
 				layer.FlipVertical ();
 
 			Workspace.Invalidate ();
@@ -458,7 +463,7 @@ namespace Pinta.Core
 				g.AppendPath(Selection.SelectionPath);
 				g.Clip ();
 
-				g.SetSource (Layers[index].Surface);
+				g.SetSource (UserLayers[index].Surface);
 				g.Paint ();
 			}
 
@@ -475,7 +480,7 @@ namespace Pinta.Core
 			foreach (var layer in GetLayersToPaint ()) {
 				var blend_op = UserBlendOps.GetBlendOp (layer.BlendMode, layer.Opacity);
 
-				pixel = blend_op.Apply (pixel, layer.Surface.GetColorBgra (x, y));
+				pixel = blend_op.Apply (pixel, layer.Surface.GetColorBgraUnchecked (x, y));
 			}
 
 			return pixel;
@@ -500,16 +505,28 @@ namespace Pinta.Core
 		{
 			List<Layer> paint = new List<Layer> ();
 
-			foreach (var layer in Layers) {
+			foreach (var layer in UserLayers) {
 				if (!layer.Hidden)
 					paint.Add (layer);
 
-				if (layer == CurrentLayer) {
-					if (!tool_layer.Hidden)
-						paint.Add (tool_layer);
+				if (layer == CurrentUserLayer) {
+					if (!ToolLayer.Hidden)
+						paint.Add (ToolLayer);
 
 					if (ShowSelectionLayer)
-						paint.Add (selection_layer);
+						paint.Add (SelectionLayer);
+				}
+
+				if (!layer.Hidden)
+				{
+					foreach (ReEditableLayer rel in layer.ReEditableLayers)
+					{
+						//Make sure that each UserLayer's ReEditableLayer is in use before adding it to the List of Layers to Paint.
+						if (rel.IsLayerSetup)
+						{
+							paint.Add(rel.Layer);
+						}
+					}
 				}
 			}
 
@@ -527,17 +544,17 @@ namespace Pinta.Core
 			return bounds;
 		}
 
-		public int IndexOf (Layer layer)
+		public int IndexOf(UserLayer layer)
 		{
-			return Layers.IndexOf (layer);
+			return UserLayers.IndexOf (layer);
 		}
 
 		// Adds a new layer above the current one
-		public void Insert (Layer layer, int index)
+		public void Insert(UserLayer layer, int index)
 		{
-			Layers.Insert (index, layer);
+			UserLayers.Insert (index, layer);
 
-			if (Layers.Count == 1)
+			if (UserLayers.Count == 1)
 				current_layer = 0;
 
 			layer.PropertyChanged += RaiseLayerPropertyChangedEvent;
@@ -552,8 +569,8 @@ namespace Pinta.Core
 				throw new InvalidOperationException ("Cannot flatten layer because current layer is the bottom layer.");
 
 			// Get our source and destination layers
-			var source = CurrentLayer;
-			var dest = Layers[current_layer - 1];
+			var source = CurrentUserLayer;
+			var dest = UserLayers[current_layer - 1];
 
 			// Blend the layers
 			var blendop = UserBlendOps.GetBlendOp (source.BlendMode, source.Opacity);
@@ -568,9 +585,9 @@ namespace Pinta.Core
 			if (current_layer == 0)
 				throw new InvalidOperationException ("Cannot move layer down because current layer is the bottom layer.");
 
-			Layer layer = CurrentLayer;
-			Layers.RemoveAt (current_layer);
-			Layers.Insert (--current_layer, layer);
+			UserLayer layer = CurrentUserLayer;
+			UserLayers.RemoveAt (current_layer);
+			UserLayers.Insert (--current_layer, layer);
 
 			PintaCore.Layers.OnSelectedLayerChanged ();
 
@@ -580,23 +597,22 @@ namespace Pinta.Core
 		// Move current layer up
 		public void MoveCurrentLayerUp ()
 		{
-			if (current_layer == Layers.Count)
+			if (current_layer == UserLayers.Count)
 				throw new InvalidOperationException ("Cannot move layer up because current layer is the top layer.");
 
-			Layer layer = CurrentLayer;
-			Layers.RemoveAt (current_layer);
-			Layers.Insert (++current_layer, layer);
+			UserLayer layer = CurrentUserLayer;
+			UserLayers.RemoveAt (current_layer);
+			UserLayers.Insert (++current_layer, layer);
 
 			PintaCore.Layers.OnSelectedLayerChanged ();
 
 			Workspace.Invalidate ();
 		}
 		
-		public void ResetSelectionPath()
+		public void ResetSelectionPaths()
 		{
-			Selection.DisposeSelectionPreserve();
-
 			Selection.ResetSelection(selection_layer.Surface, ImageSize);
+			PreviousSelection.ResetSelection(selection_layer.Surface, ImageSize);
 
 			ShowSelection = false;
 		}
@@ -604,6 +620,9 @@ namespace Pinta.Core
 		/// <summary>
 		/// Resizes the canvas.
 		/// </summary>
+		/// <param name="width">The new width of the canvas.</param>
+		/// <param name="height">The new height of the canvas.</param>
+		/// <param name="anchor">Direction in which to adjust the canvas</param>
 		/// <param name='compoundAction'>
 		/// Optionally, the history item for resizing the canvas can be added to
 		/// a CompoundHistoryItem if it is part of a larger action (e.g. pasting an image).
@@ -626,7 +645,7 @@ namespace Pinta.Core
 
 			ImageSize = new Gdk.Size (width, height);
 
-			foreach (var layer in Layers)
+			foreach (var layer in UserLayers)
 				layer.ResizeCanvas (width, height, anchor);
 
 			hist.FinishSnapshotOfImage ();
@@ -637,7 +656,7 @@ namespace Pinta.Core
 				Workspace.History.PushNewItem (hist);
 			}
 
-			ResetSelectionPath ();
+			ResetSelectionPaths ();
 
 			Workspace.Scale = scale;
 		}
@@ -658,14 +677,14 @@ namespace Pinta.Core
 
 			ImageSize = new Gdk.Size (width, height);
 
-			foreach (var layer in Layers)
+			foreach (var layer in UserLayers)
 				layer.Resize (width, height);
 
 			hist.FinishSnapshotOfImage ();
 
 			Workspace.History.PushNewItem (hist);
 
-			ResetSelectionPath ();
+			ResetSelectionPaths ();
 
 			Workspace.Scale = scale;
 		}
@@ -691,7 +710,7 @@ namespace Pinta.Core
 		/// </summary>
 		private void RotateImage (double angle)
 		{
-			foreach (var layer in Layers)
+			foreach (var layer in UserLayers)
 			{
 				layer.Rotate (angle);
 			}
@@ -709,7 +728,7 @@ namespace Pinta.Core
 			return PintaCore.Actions.File.RaiseSaveDocument (this, saveAs);
 		}
 
-		public void SetCurrentLayer (int i)
+		public void SetCurrentUserLayer (int i)
 		{
 			// Ensure that the current tool's modifications are finalized before
 			// switching layers.
@@ -719,9 +738,168 @@ namespace Pinta.Core
 			PintaCore.Layers.OnSelectedLayerChanged ();
 		}
 
-		public void SetCurrentLayer (Layer layer)
+		public void SetCurrentUserLayer(UserLayer layer)
 		{
-			SetCurrentLayer (Layers.IndexOf (layer));
+			SetCurrentUserLayer (UserLayers.IndexOf (layer));
+		}
+
+		/// <summary>
+		/// Pastes an image from the clipboard.
+		/// </summary>
+		/// <param name="toNewLayer">Set to TRUE to paste into a
+		/// new layer.  Otherwise, will paste to the current layer.</param>
+		/// <param name="x">Optional. Location within image to paste to.
+		/// Position will be adjusted if pasted image would hang
+		/// over right or bottom edges of canvas.</param>
+		/// <param name="y">Optional. Location within image to paste to.
+		/// Position will be adjusted if pasted image would hang
+		/// over right or bottom edges of canvas.</param>
+		public void Paste (bool toNewLayer, int x = 0, int y = 0)
+		{
+			// Create a compound history item for recording several
+			// operations so that they can all be undone/redone together.
+			CompoundHistoryItem paste_action;
+			if (toNewLayer)
+			{
+				paste_action = new CompoundHistoryItem (Stock.Paste, Catalog.GetString ("Paste Into New Layer"));
+			}
+			else
+			{
+				paste_action = new CompoundHistoryItem (Stock.Paste, Catalog.GetString ("Paste"));
+			}
+
+			Gtk.Clipboard cb = Gtk.Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
+
+			// See if the current tool wants to handle the paste
+			// operation (e.g., the text tool could paste text)
+			if (!toNewLayer)
+			{
+				if (PintaCore.Tools.CurrentTool.TryHandlePaste (cb))
+					return;
+			}
+
+			PintaCore.Tools.Commit ();
+
+			// Don't dispose this, as we're going to give it to the history
+			Gdk.Pixbuf cbImage = null;
+
+			if (cb.WaitIsImageAvailable ()) {
+				cbImage = cb.WaitForImage ();
+			}
+
+			if (cbImage == null)
+			{
+				ShowClipboardEmptyDialog();
+				return;
+			}
+
+			Gdk.Size canvas_size = PintaCore.Workspace.ImageSize;
+
+			// If the image being pasted is larger than the canvas size, allow the user to optionally resize the canvas
+			if (cbImage.Width > canvas_size.Width || cbImage.Height > canvas_size.Height)
+			{
+				ResponseType response = ShowExpandCanvasDialog ();
+			
+				if (response == ResponseType.Accept)
+				{
+					PintaCore.Workspace.ResizeCanvas (cbImage.Width, cbImage.Height,
+					Pinta.Core.Anchor.Center, paste_action);
+					PintaCore.Actions.View.UpdateCanvasScale ();
+				}
+				else if (response == ResponseType.Cancel || response == ResponseType.DeleteEvent)
+				{
+					return;
+				}
+			}
+
+			// If the pasted image would fall off bottom- or right-
+			// side of image, adjust paste position
+			x = Math.Max (0, Math.Min (x, canvas_size.Width - cbImage.Width));
+			y = Math.Max (0, Math.Min (y, canvas_size.Height - cbImage.Height));
+
+			// If requested, create a new layer, make it the current
+			// layer and record it's creation in the history
+			if (toNewLayer)
+			{
+				UserLayer l = AddNewLayer (string.Empty);
+				SetCurrentUserLayer (l);
+				paste_action.Push (new AddLayerHistoryItem ("Menu.Layers.AddNewLayer.png", Catalog.GetString ("Add New Layer"), UserLayers.IndexOf (l)));
+			}
+
+			// Copy the paste to the temp layer, which should be at least the size of this document.
+			CreateSelectionLayer (Math.Max(ImageSize.Width, cbImage.Width),
+			                      Math.Max(ImageSize.Height, cbImage.Height));
+			ShowSelectionLayer = true;
+			
+			using (Cairo.Context g = new Cairo.Context (SelectionLayer.Surface))
+			{
+				g.DrawPixbuf (cbImage, new Cairo.Point (0, 0));
+			}
+
+			SelectionLayer.Transform.InitIdentity();
+			SelectionLayer.Transform.Translate (x, y);
+
+			PintaCore.Tools.SetCurrentTool (Catalog.GetString ("Move Selected Pixels"));
+			
+			DocumentSelection old_selection = Selection.Clone();
+			bool old_show_selection = ShowSelection;
+
+			Selection.CreateRectangleSelection (SelectionLayer.Surface,
+			                                    new Cairo.Rectangle (x, y, cbImage.Width, cbImage.Height));
+			ShowSelection = true;
+
+			Workspace.Invalidate ();
+
+			paste_action.Push (new PasteHistoryItem (cbImage, old_selection, old_show_selection));
+			History.PushNewItem (paste_action);
+		}
+
+		private ResponseType ShowExpandCanvasDialog ()
+		{
+			const string markup = "<span weight=\"bold\" size=\"larger\">{0}</span>\n\n{1}";
+			string primary = Catalog.GetString ("Image larger than canvas");
+			string secondary = Catalog.GetString ("The image being pasted is larger than the canvas size. What would you like to do?");
+			string message = string.Format (markup, primary, secondary);
+
+			var enlarge_dialog = new MessageDialog (PintaCore.Chrome.MainWindow, DialogFlags.Modal, MessageType.Question, ButtonsType.None, message);
+			enlarge_dialog.AddButton (Catalog.GetString ("Expand canvas"), ResponseType.Accept);
+			enlarge_dialog.AddButton (Catalog.GetString ("Don't change canvas size"), ResponseType.Reject);
+			enlarge_dialog.AddButton (Stock.Cancel, ResponseType.Cancel);
+			enlarge_dialog.DefaultResponse = ResponseType.Accept;
+
+			ResponseType response = (ResponseType)enlarge_dialog.Run ();
+
+			enlarge_dialog.Destroy ();
+
+			return response;
+		}
+
+		public static void ShowClipboardEmptyDialog()
+		{
+			var primary = Catalog.GetString ("Image cannot be pasted");
+			var secondary = Catalog.GetString ("The clipboard does not contain an image.");
+			var markup = "<span weight=\"bold\" size=\"larger\">{0}</span>\n\n{1}\n";
+			markup = string.Format (markup, primary, secondary);
+
+			var md = new MessageDialog (Pinta.Core.PintaCore.Chrome.MainWindow, DialogFlags.Modal,
+						    MessageType.Error, ButtonsType.None, true,
+						    markup);
+
+			md.AddButton (Stock.Ok, ResponseType.Yes);
+
+			md.Run ();
+			md.Destroy ();
+		}
+
+		/// <summary>
+		/// Signal to the TextTool that an ImageSurface was cloned.
+		/// </summary>
+		public void SignalSurfaceCloned()
+		{
+			if (LayerCloned != null)
+			{
+				LayerCloned();
+			}
 		}
 		#endregion
 
@@ -749,6 +927,7 @@ namespace Pinta.Core
 		#region Public Events
 		public event EventHandler IsDirtyChanged;
 		public event EventHandler Renamed;
+		public event LayerCloneEvent LayerCloned;
 		#endregion
 	}
 }
