@@ -44,7 +44,7 @@ namespace Pinta.Core
 		internal DocumentWorkspace (Document document)
 		{
 			this.document = document;
-			History = new DocumentWorkspaceHistory (document, this);
+			History = new DocumentWorkspaceHistory (document);
 		}
 
 		#region Public Properties
@@ -96,11 +96,27 @@ namespace Pinta.Core
 			get { return (double)CanvasSize.Width / (double)document.ImageSize.Width; }
 			set {
 				if (value != (double)CanvasSize.Width / (double)document.ImageSize.Width || value != (double)CanvasSize.Height / (double)document.ImageSize.Height) {
+					if (document.ImageSize.Width == 0)
+					{
+						document.ImageSize = new Size(1, document.ImageSize.Height);
+					}
+
+					if (document.ImageSize.Height == 0)
+					{
+						document.ImageSize = new Size(document.ImageSize.Width, 1);
+					}
+
 					int new_x = Math.Max ((int)(document.ImageSize.Width * value), 1);
-					int new_y = Math.Max ((int)((new_x * document.ImageSize.Height) / document.ImageSize.Width), 1);
+					int new_y = Math.Max ((int)(((long)new_x * document.ImageSize.Height) / document.ImageSize.Width), 1);
 
 					CanvasSize = new Gdk.Size (new_x, new_y);
 					Invalidate ();
+
+					if (PintaCore.Tools.CurrentTool.CursorChangesOnZoom)
+					{
+						//The current tool's cursor changes when the zoom changes.
+						PintaCore.Tools.CurrentTool.SetCursor(PintaCore.Tools.CurrentTool.DefaultCursor);
+					}
 				}
 			}
 		}
@@ -113,10 +129,23 @@ namespace Pinta.Core
 			PintaCore.Workspace.OnCanvasInvalidated (new CanvasInvalidatedEventArgs ());
 		}
 
-		public void Invalidate (Gdk.Rectangle rect)
+		/// <summary>
+		/// Repaints a rectangle region on the canvas.
+		/// </summary>
+		/// <param name='canvasRect'>
+		/// The rectangle region of the canvas requiring repainting
+		/// </param>
+		public void Invalidate (Gdk.Rectangle canvasRect)
 		{
-			rect = new Gdk.Rectangle ((int)((rect.X) * Scale + Offset.X), (int)((rect.Y) * Scale + Offset.Y), (int)(rect.Width * Scale), (int)(rect.Height * Scale));
-			PintaCore.Workspace.OnCanvasInvalidated (new CanvasInvalidatedEventArgs (rect));
+			Cairo.PointD canvasTopLeft = new Cairo.PointD(canvasRect.Left, canvasRect.Top);
+			Cairo.PointD canvasBtmRight = new Cairo.PointD(canvasRect.Right + 1, canvasRect.Bottom + 1);
+
+			Cairo.PointD winTopLeft = CanvasPointToWindow(canvasTopLeft.X, canvasTopLeft.Y);
+			Cairo.PointD winBtmRight = CanvasPointToWindow(canvasBtmRight.X, canvasBtmRight.Y);
+
+			Gdk.Rectangle winRect = Utility.PointsToRectangle(winTopLeft, winBtmRight, false).ToGdkRectangle();
+
+			PintaCore.Workspace.OnCanvasInvalidated (new CanvasInvalidatedEventArgs (winRect));
 		}
 
 		/// <summary>
@@ -124,8 +153,7 @@ namespace Pinta.Core
 		/// </summary>
 		public bool IsPartiallyOffscreen (Gdk.Rectangle rect)
 		{
-			return (rect.IsEmpty || rect.Left < 0 || rect.Top < 0 ||
-			        rect.Bottom > CanvasSize.Height || rect.Right > CanvasSize.Width);
+			return (rect.IsEmpty || rect.Left < 0 || rect.Top < 0);
 		}
 
 		public bool PointInCanvas (Cairo.PointD point)
@@ -155,9 +183,38 @@ namespace Pinta.Core
 			view.Vadjustment.Value = Utility.Clamp (dy + view.Vadjustment.Value, view.Vadjustment.Lower, view.Vadjustment.Upper - view.Vadjustment.PageSize);
 		}
 
+		/// <summary>
+		/// Converts a point from window coordinates to canvas coordinates
+		/// </summary>
+		/// <param name='x'>
+		/// The X coordinate of the window point
+		/// </param>
+		/// <param name='y'>
+		/// The Y coordinate of the window point
+		/// </param>
 		public Cairo.PointD WindowPointToCanvas (double x, double y)
 		{
-			return new Cairo.PointD (Math.Floor ((x - Offset.X) / PintaCore.Workspace.Scale), Math.Floor ((y - Offset.Y) / PintaCore.Workspace.Scale));
+			ScaleFactor sf = new ScaleFactor (PintaCore.Workspace.ImageSize.Width,
+			                                  PintaCore.Workspace.CanvasSize.Width);
+			Cairo.PointD pt = sf.ScalePoint (new Cairo.PointD (x - Offset.X, y - Offset.Y));
+			return new Cairo.PointD((int)pt.X, (int)pt.Y);
+		}
+
+		/// <summary>
+		/// Converts a point from canvas coordinates to window coordinates
+		/// </summary>
+		/// <param name='x'>
+		/// The X coordinate of the canvas point
+		/// </param>
+		/// <param name='y'>
+		/// The Y coordinate of the canvas point
+		/// </param>
+		public Cairo.PointD CanvasPointToWindow (double x, double y)
+		{
+			ScaleFactor sf = new ScaleFactor (PintaCore.Workspace.ImageSize.Width,
+			                                  PintaCore.Workspace.CanvasSize.Width);
+			Cairo.PointD pt = sf.UnscalePoint (new Cairo.PointD (x, y));
+			return new Cairo.PointD(pt.X + Offset.X, pt.Y + Offset.Y);
 		}
 
 		public void ZoomIn ()
@@ -194,7 +251,7 @@ namespace Pinta.Core
 			else
 				ratio = document.ImageSize.Height / rect.Height;
 			
-			(PintaCore.Actions.View.ZoomComboBox.ComboBox as Gtk.ComboBoxEntry).Entry.Text = String.Format ("{0:F}%", ratio * 100.0);
+			(PintaCore.Actions.View.ZoomComboBox.ComboBox as Gtk.ComboBoxEntry).Entry.Text = ViewActions.ToPercent (ratio);
 			Gtk.Main.Iteration (); //Force update of scrollbar upper before recenter
 			RecenterView (rect.X + rect.Width / 2, rect.Y + rect.Height / 2);
 		}
@@ -233,9 +290,13 @@ namespace Pinta.Core
 				
 				Predicate<string> UpdateZoomLevel = zoomInList =>
 				{
+					double zoom_level;
+					if (!ViewActions.TryParsePercent (zoomInList, out zoom_level))
+						return false;
+
 					switch (zoomType) {
 					case ZoomType.ZoomIn:
-						if (zoomInList == Catalog.GetString ("Window") || int.Parse (zoomInList.Trim ('%')) <= zoom) {
+						if (zoomInList == Catalog.GetString ("Window") || zoom_level <= zoom) {
 							PintaCore.Actions.View.ZoomComboBox.ComboBox.Active = i - 1;
 							return true;
 						}
@@ -246,7 +307,7 @@ namespace Pinta.Core
 						if (zoomInList == Catalog.GetString ("Window"))
 							return true;
 						
-						if (int.Parse (zoomInList.Trim ('%')) < zoom) {
+						if (zoom_level < zoom) {
 							PintaCore.Actions.View.ZoomComboBox.ComboBox.Active = i;
 							return true;
 						}
