@@ -36,6 +36,7 @@
 using System;
 using Cairo;
 using System.Collections.Generic;
+using System.Runtime.InteropServices;
 
 namespace Pinta.Core
 {
@@ -49,7 +50,7 @@ namespace Pinta.Core
 		{
 			// Put it on a pixel line
 			if (lineWidth == 1)
-				r = new Rectangle (r.X - 0.5, r.Y - 0.5, r.Width, r.Height);
+				r = new Rectangle (r.X + 0.5, r.Y + 0.5, r.Width-1, r.Height-1);
 
 			g.Save ();
 
@@ -59,11 +60,11 @@ namespace Pinta.Core
 			g.LineTo (r.X, r.Y + r.Height);
 			g.LineTo (r.X, r.Y);
 
-			g.Color = color;
+			g.SetSourceColor (color);
 			g.LineWidth = lineWidth;
 			g.LineCap = LineCap.Square;
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 			g.Stroke ();
 
 			g.Restore ();
@@ -71,6 +72,31 @@ namespace Pinta.Core
 			return dirty;
 		}
 
+		public static Rectangle DrawFullRectangle(this Context g, Rectangle r, Color color, int lineWidth)
+		{
+			// Put it on a pixel line
+			if (lineWidth == 1)
+				r = new Rectangle(r.X + 0.5, r.Y + 0.5, r.Width - 1, r.Height - 1);
+
+			g.Save();
+
+			g.MoveTo(r.X - (double)lineWidth / 2d, r.Y);
+			g.LineTo(r.X + r.Width, r.Y);
+			g.LineTo(r.X + r.Width, r.Y + r.Height);
+			g.LineTo(r.X, r.Y + r.Height);
+			g.LineTo(r.X, r.Y);
+
+			g.SetSourceColor (color);
+			g.LineWidth = lineWidth;
+
+			Rectangle dirty = g.FixedStrokeExtents();
+			g.Stroke();
+
+			g.Restore();
+
+			return dirty;
+		}
+
 		public static Path CreateRectanglePath (this Context g, Rectangle r)
 		{
 			g.Save ();
@@ -97,9 +123,9 @@ namespace Pinta.Core
 			g.LineTo (r.X, r.Y + r.Height);
 			g.LineTo (r.X, r.Y);
 
-			g.Color = color;
+			g.SetSourceColor (color);
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Fill ();
 			g.Restore ();
@@ -117,9 +143,9 @@ namespace Pinta.Core
 			g.LineTo (r.X, r.Y + r.Height);
 			g.LineTo (r.X, r.Y);
 
-			g.Pattern = pattern;
+			g.SetSource (pattern);
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 			g.Fill ();
 
 			g.Restore ();
@@ -129,18 +155,15 @@ namespace Pinta.Core
 
 		public static Rectangle DrawPolygonal (this Context g, PointD[] points, Color color)
 		{
-			Random rand = new Random ();
-
 			g.Save ();
 			g.MoveTo (points[0]);
 			foreach (var point in points) {
-				g.LineTo (point.X - rand.NextDouble () * 0, point.Y);
-				//g.Stroke();
+				g.LineTo (point.X, point.Y);
 			}
 
-			g.Color = color;
+			g.SetSourceColor (color);
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 			g.Stroke ();
 
 			g.Restore ();
@@ -156,9 +179,9 @@ namespace Pinta.Core
 			foreach (var point in points)
 				g.LineTo (point);
 
-			g.Color = color;
+			g.SetSourceColor (color);
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 			g.Fill ();
 
 			g.Restore ();
@@ -185,14 +208,14 @@ namespace Pinta.Core
 			g.LineTo (x, y + r.Height);
 			g.LineTo (x, y);
 
-			g.Color = fill;
+			g.SetSourceColor (fill);
 			g.FillPreserve ();
 
-			g.Color = stroke;
+			g.SetSourceColor (stroke);
 			g.LineWidth = lineWidth;
 			g.LineCap = LineCap.Square;
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Stroke ();
 			g.Restore ();
@@ -200,6 +223,40 @@ namespace Pinta.Core
 			return dirty;
 		}
 
+		public static Rectangle FillStrokedFullRectangle(this Context g, Rectangle r, Color fill, Color stroke, int lineWidth)
+		{
+			double x = r.X;
+			double y = r.Y;
+
+			g.Save();
+
+			// Put it on a pixel line
+			if (lineWidth == 1)
+			{
+				x += 0.5;
+				y += 0.5;
+			}
+
+			g.MoveTo(x - (double)lineWidth / 2d, y);
+			g.LineTo(x + r.Width, y);
+			g.LineTo(x + r.Width, y + r.Height);
+			g.LineTo(x, y + r.Height);
+			g.LineTo(x, y);
+
+			g.SetSourceColor (fill);
+			g.FillPreserve();
+
+			g.SetSourceColor (stroke);
+			g.LineWidth = lineWidth;
+
+			Rectangle dirty = g.FixedStrokeExtents();
+
+			g.Stroke();
+			g.Restore();
+
+			return dirty;
+		}
+
 		public static Rectangle DrawEllipse (this Context g, Rectangle r, Color color, int lineWidth)
 		{
 			double rx = r.Width / 2;
@@ -219,10 +276,10 @@ namespace Pinta.Core
 
 			g.ClosePath ();
 
-			g.Color = color;
+			g.SetSourceColor (color);
 			g.LineWidth = lineWidth;
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Stroke ();
 			g.Restore ();
@@ -249,9 +306,9 @@ namespace Pinta.Core
 
 			g.ClosePath ();
 
-			g.Color = color;
+			g.SetSourceColor (color);
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Fill ();
 			g.Restore ();
@@ -304,13 +361,13 @@ namespace Pinta.Core
 
 			g.ClosePath ();
 
-			g.Color = fill;
+			g.SetSourceColor (fill);
 			g.FillPreserve ();
 
-			g.Color = stroke;
+			g.SetSourceColor (stroke);
 			g.LineWidth = lineWidth;
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Stroke ();
 			g.Restore ();
@@ -335,13 +392,13 @@ namespace Pinta.Core
 			g.Arc (r.X + radius, r.Y + r.Height - radius, radius, Math.PI / 2, Math.PI);
 			g.ClosePath ();
 
-			g.Color = fill;
+			g.SetSourceColor (fill);
 			g.FillPreserve ();
 
-			g.Color = stroke;
+			g.SetSourceColor (stroke);
 			g.LineWidth = lineWidth;
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Stroke ();
 			g.Restore ();
@@ -366,9 +423,9 @@ namespace Pinta.Core
 			g.Arc (r.X + radius, r.Y + r.Height - radius, radius, Math.PI / 2, Math.PI);
 			g.ClosePath ();
 
-			g.Color = fill;
+			g.SetSourceColor (fill);
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Fill ();
 			g.Restore ();
@@ -380,7 +437,7 @@ namespace Pinta.Core
 		{
 			g.Save ();
 
-			g.Color = color;
+			g.SetSourceColor (color);
 
 			foreach (Gdk.Rectangle r in region.GetRectangles ()) {
 				g.MoveTo (r.X, r.Y);
@@ -389,9 +446,9 @@ namespace Pinta.Core
 				g.LineTo (r.X, r.Y + r.Height);
 				g.LineTo (r.X, r.Y);
 
-				g.Color = color;
+				g.SetSourceColor (color);
 
-				g.StrokeExtents ();
+				g.FixedStrokeExtents ();
 				g.Fill ();
 			}
 
@@ -406,10 +463,10 @@ namespace Pinta.Core
 
 			g.AppendPath (p);
 
-			g.Color = stroke;
+			g.SetSourceColor (stroke);
 			g.LineWidth = lineWidth;
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 
 			g.Stroke ();
 			g.Restore ();
@@ -456,19 +513,21 @@ namespace Pinta.Core
 		public static Rectangle DrawLine (this Context g, PointD p1, PointD p2, Color color, int lineWidth)
 		{
 			// Put it on a pixel line
-			if (lineWidth == 1)
-				p1 = new PointD (p1.X - 0.5, p1.Y - 0.5);
+			if (lineWidth == 1){
+				p1 = new PointD (p1.X + 0.5, p1.Y + 0.5);
+				p2 = new PointD (p2.X + 0.5, p2.Y + 0.5);
+			}
 
 			g.Save ();
 
 			g.MoveTo (p1.X, p1.Y);
 			g.LineTo (p2.X, p2.Y);
 
-			g.Color = color;
+			g.SetSourceColor (color);
 			g.LineWidth = lineWidth;
 			g.LineCap = LineCap.Square;
 
-			Rectangle dirty = g.StrokeExtents ();
+			Rectangle dirty = g.FixedStrokeExtents ();
 			g.Stroke ();
 
 			g.Restore ();
@@ -476,6 +535,52 @@ namespace Pinta.Core
 			return dirty;
 		}
 
+		/// <summary>
+		/// Computes a bounding box in user coordinates covering the
+		/// area that would be affected by a call to Context.Stroke()
+		/// using the current stroke parameters.
+		/// 
+		/// The rectangle returned by Cairo.Context.StrokeExtents()
+		/// incorrectly specifies the X and Y coordinates of the
+		/// bottom-right corner of the Rectangle in the width and
+		/// height members. This method corrects the rectangle to
+		/// contain the width and height in the width and height members.
+		/// 
+		/// This can be removed once we port to GTK3.
+		/// </summary>
+		/// <returns>
+		/// The rectangle describing the area that would be
+		/// affected.
+		/// </returns>
+		public static Rectangle FixedStrokeExtents (this Context g)
+		{
+			double x1, y1, x2, y2;
+			cairo_stroke_extents (g.Handle, out x1, out y1, out x2, out y2);
+			return new Rectangle (x1, y1, x2 - x1, y2 - y1);
+		}
+
+		/// <summary>
+		/// The Pattern property is now deprecated in favour of the SetSource (pattern) method,
+		/// but that method doesn't exist in older versions of Mono.Cairo. This extension method
+		/// provides an implementation of that functionality.
+		/// 
+		/// This can be removed once we port to GTK3.
+		/// </summary>
+		public static void SetSource (this Context g, Pattern source)
+		{
+#pragma warning disable 612
+			cairo_set_source (g.Handle, source.Pointer);
+#pragma warning restore 612
+		}
+
+		private const string CairoLib = "libcairo-2.dll";
+
+		[DllImport (CairoLib, CallingConvention=CallingConvention.Cdecl)]
+		private static extern void cairo_stroke_extents (IntPtr cr, out double x1, out double y1, out double x2, out double y2);
+
+		[DllImport (CairoLib, CallingConvention=CallingConvention.Cdecl)]
+		private static extern void cairo_set_source (IntPtr cr, IntPtr pattern);
+
 		private static Pango.Style CairoToPangoSlant (FontSlant slant)
 		{
 			switch (slant) {
@@ -498,7 +603,7 @@ namespace Pinta.Core
 			g.Save ();
 
 			g.MoveTo (p.X, p.Y);
-			g.Color = color;
+			g.SetSourceColor (color);
 			g.Antialias = antiAliasing ? Antialias.Subpixel : Antialias.None;
 
 			Pango.Layout layout = Pango.CairoHelper.CreateLayout (g);
@@ -540,12 +645,12 @@ namespace Pinta.Core
 			if (mode == GradientColorMode.Color) {
 				gradient.AddColorStop (0, c1);
 				gradient.AddColorStop (1, c2);
-				g.Source = gradient;
+				g.SetSource (gradient);
 				g.Paint ();
 			} else if (mode == GradientColorMode.Transparency) {
 				gradient.AddColorStop (0, new Color (0, 0, 0, 1));
 				gradient.AddColorStop (1, new Color (0, 0, 0, 0));
-				g.Source = new SurfacePattern (oldsurface);
+				g.SetSource (new SurfacePattern (oldsurface));
 				g.Mask (gradient);
 			}
 
@@ -562,13 +667,13 @@ namespace Pinta.Core
 				gradient.AddColorStop (0, c1);
 				gradient.AddColorStop (0.5, c2);
 				gradient.AddColorStop (1, c1);
-				g.Source = gradient;
+				g.SetSource (gradient);
 				g.Paint ();
 			} else if (mode == GradientColorMode.Transparency) {
 				gradient.AddColorStop (0, new Color (0, 0, 0, 1));
 				gradient.AddColorStop (0.5, new Color (0, 0, 0, 0));
 				gradient.AddColorStop (1, new Color (0, 0, 0, 1));
-				g.Source = new SurfacePattern (oldsurface);
+				g.SetSource (new SurfacePattern (oldsurface));
 				g.Mask (gradient);
 			}
 
@@ -584,17 +689,23 @@ namespace Pinta.Core
 			if (mode == GradientColorMode.Color) {
 				gradient.AddColorStop (0, c1);
 				gradient.AddColorStop (1, c2);
-				g.Source = gradient;
+				g.SetSource (gradient);
 				g.Paint ();
 			} else if (mode == GradientColorMode.Transparency) {
 				gradient.AddColorStop (0, new Color (0, 0, 0, 1));
 				gradient.AddColorStop (1, new Color (0, 0, 0, 0));
-				g.Source = new SurfacePattern (oldsurface);
+				g.SetSource (new SurfacePattern (oldsurface));
 				g.Mask (gradient);
 			}
 
 			g.Restore ();
 		}
+
+		// The Color property is deprecated, so use this extension method until SetSourceColor is officially available everywhere.
+		public static void SetSourceColor (this Context g, Color c)
+		{
+			g.SetSourceRGBA (c.R, c.G, c.B, c.A);
+		}
 		#endregion
 
 		public static double Distance (this PointD s, PointD e)
@@ -639,7 +750,8 @@ namespace Pinta.Core
 
 		public static Gdk.Rectangle ToGdkRectangle (this Cairo.Rectangle r)
 		{
-			return new Gdk.Rectangle ((int)r.X, (int)r.Y, (int)r.Width, (int)r.Height);
+			return new Gdk.Rectangle ((int)Math.Floor (r.X), (int)Math.Floor (r.Y),
+			                          (int)Math.Ceiling (r.Width), (int)Math.Ceiling (r.Height));
 		}
 
 		public static bool ContainsPoint (this Cairo.Rectangle r, double x, double y)
@@ -660,21 +772,21 @@ namespace Pinta.Core
 
 		public unsafe static Gdk.Pixbuf ToPixbuf (this Cairo.ImageSurface surfSource)
 		{
-			Cairo.ImageSurface surf = surfSource.Clone ();
-			surf.Flush ();
+			using (Cairo.ImageSurface surf = surfSource.Clone ()) {
+				surf.Flush ();
 
-			ColorBgra* dstPtr = (ColorBgra*)surf.DataPtr;
-			int len = surf.Data.Length / 4;
+				ColorBgra* dstPtr = (ColorBgra*)surf.DataPtr;
+				int len = surf.Data.Length / 4;
 
-			for (int i = 0; i < len; i++) {
-				if (dstPtr->A != 0)
-					*dstPtr = (ColorBgra.FromBgra (dstPtr->R, dstPtr->G, dstPtr->B, dstPtr->A));
-				dstPtr++;
+				for (int i = 0; i < len; i++) {
+					if (dstPtr->A != 0)
+						*dstPtr = (ColorBgra.FromBgra (dstPtr->R, dstPtr->G, dstPtr->B, dstPtr->A));
+					dstPtr++;
+				}
+
+				Gdk.Pixbuf pb = new Gdk.Pixbuf (surf.Data, true, 8, surf.Width, surf.Height, surf.Stride);
+				return pb;
 			}
-
-			Gdk.Pixbuf pb = new Gdk.Pixbuf (surf.Data, true, 8, surf.Width, surf.Height, surf.Stride);
-			(surf as IDisposable).Dispose ();
-			return pb;
 		}
 
 		public unsafe static Color GetPixel (this Cairo.ImageSurface surf, int x, int y)
@@ -710,7 +822,7 @@ namespace Pinta.Core
 			dstPtr->A = (byte)(color.A * 255);
 		}
 
-		public unsafe static ColorBgra GetColorBgra (this Cairo.ImageSurface surf, int x, int y)
+		public unsafe static ColorBgra GetColorBgraUnchecked (this Cairo.ImageSurface surf, int x, int y)
 		{
 			ColorBgra* dstPtr = (ColorBgra*)surf.DataPtr;
 
@@ -721,6 +833,9 @@ namespace Pinta.Core
 
 		public unsafe static void SetColorBgra (this Cairo.ImageSurface surf, ColorBgra color, int x, int y)
 		{
+			if (x > surf.Width || y > surf.Height)
+				throw new ArgumentOutOfRangeException ("Invalid canvas coordinates");
+
 			ColorBgra* dstPtr = (ColorBgra*)surf.DataPtr;
 
 			dstPtr += (x) + (y * surf.Width);
@@ -728,7 +843,11 @@ namespace Pinta.Core
 			*dstPtr = color;
 		}
 
-		public unsafe static void SetColorBgra (this Cairo.ImageSurface surf, ColorBgra* surfDataPtr, int surfWidth, ColorBgra color, int x, int y)
+		/// <summary>
+		/// For performance reasons, this method does not check that the x and y coordinates are
+		/// within the bounds of the image.
+		/// </summary>
+		public unsafe static void SetColorBgraUnchecked (this Cairo.ImageSurface surf, ColorBgra* surfDataPtr, int surfWidth, ColorBgra color, int x, int y)
 		{
 			ColorBgra* dstPtr = surfDataPtr;
 
@@ -737,7 +856,7 @@ namespace Pinta.Core
 			*dstPtr = color;
 		}
 
-		public unsafe static ColorBgra GetColorBgra (this Cairo.ImageSurface surf, ColorBgra* surfDataPtr, int surfWidth, int x, int y)
+		public unsafe static ColorBgra GetColorBgraUnchecked (this Cairo.ImageSurface surf, ColorBgra* surfDataPtr, int surfWidth, int x, int y)
 		{
 			ColorBgra* dstPtr = surfDataPtr;
 
@@ -799,6 +918,9 @@ namespace Pinta.Core
 
 		public static ImageSurface Clone (this ImageSurface surf)
 		{
+			if (PintaCore.Workspace.HasOpenDocuments)
+				PintaCore.Workspace.ActiveDocument.SignalSurfaceCloned ();
+
 			ImageSurface newsurf = new ImageSurface (surf.Format, surf.Width, surf.Height);
 
 			using (Context g = new Context (newsurf)) {
@@ -818,10 +940,10 @@ namespace Pinta.Core
 
 			for (int x = 0; x < width; x++)
 				for (int y = 0; y < surf.Height; y++) {
-					int a = (int)surf.GetColorBgra (ptr, width, x, y).A;
+					int a = (int)surf.GetColorBgraUnchecked (ptr, width, x, y).A;
 
 					if (a > 0 && a < 255) {
-						Console.WriteLine (surf.GetColorBgra (ptr, width, x, y).ToString ());
+						Console.WriteLine (surf.GetColorBgraUnchecked (ptr, width, x, y).ToString ());
 						ret = true;
 					}
 				}
@@ -854,13 +976,13 @@ namespace Pinta.Core
 			Rectangle rect;
 
 			using (Context g = new Context (PintaCore.Layers.CurrentLayer.Surface)) {
-				g.AppendPath (PintaCore.Workspace.ActiveDocument.Selection.SelectionPath);
+				g.AppendPath (path);
 
 				// We don't want the bounding box to include a stroke width 
 				// of 1, but setting it to 0 returns an empty rectangle.  Set
 				// it to a sufficiently small width and rounding takes care of it
 				g.LineWidth = .01;
-				rect = g.StrokeExtents ();
+				rect = g.FixedStrokeExtents ();
 			}
 
 			int x = (int)Math.Round (rect.X);
@@ -868,7 +990,12 @@ namespace Pinta.Core
 			int w = (int)Math.Round (rect.Width);
 			int h = (int)Math.Round (rect.Height);
 
-			return new Gdk.Rectangle (x, y, w - x, h - y);
+			return new Gdk.Rectangle (x, y, w, h);
+		}
+
+		public static Color Clone(this Color color)
+		{
+			return new Color(color.R, color.G, color.B, color.A);
 		}
 
 		public static Gdk.Color ToGdkColor (this Cairo.Color color)
@@ -896,9 +1023,25 @@ namespace Pinta.Core
 			return rect.X + rect.Width;
 		}
 
+		/// <summary>
+		/// Creates a copy of the rectangle but with the side edges
+		/// moved out from the center by dx, and the top and bottom
+		/// edges moved out from the center by dy.  Edges are moved
+		/// inwards when dx or dy are negative.
+		/// </summary>
+		/// <param name="rect">The source rectangle.</param>
+		/// <param name="dx">Distance to move sides from center.</param>
+		/// <param name="dy">Distance to move top and bottom from center.</param>
+		/// <returns>The inflated (or deflated) rectangle.</returns>
+		public static Rectangle Inflate (this Rectangle rect, int dx, int dy)
+		{
+			return new Rectangle(rect.X - dx, rect.Y - dy, rect.Width + 2 * dx, rect.Height + 2 * dy);
+		}
+
 		/// <summary>
 		/// Determines if the requested pixel coordinate is within bounds.
 		/// </summary>
+		/// <param name="surf">The image surface to check against.</param>
 		/// <param name="x">The x coordinate.</param>
 		/// <param name="y">The y coordinate.</param>
 		/// <returns>true if (x,y) is in bounds, false if it's not.</returns>
@@ -1411,5 +1554,95 @@ namespace Pinta.Core
 		{
 			return point.X == 0 && point.Y == 0;
 		}
+
+		public static void TransformPoint(this Matrix matrix, ref PointD point)
+		{
+			double x = point.X;
+			double y = point.Y;
+
+			matrix.TransformPoint(ref x, ref y);
+
+			point.X = x;
+			point.Y = y;
+		}
+
+		public static void InitMatrix(this Matrix matrix, Matrix source)
+		{
+			matrix.X0 = source.X0;
+			matrix.Xx = source.Xx;
+			matrix.Xy = source.Xy;
+
+			matrix.Y0 = source.Y0;
+			matrix.Yx = source.Yx;
+			matrix.Yy = source.Yy;
+		}
+
+		public static void InitRectToRect(this Matrix matrix, Rectangle src, Rectangle dst)
+		{
+			matrix.InitIdentity();
+
+			matrix.Xx = dst.Width / src.Width;
+			matrix.Yy = dst.Height / src.Height;
+			matrix.X0 = dst.X - (matrix.Xx * src.X);
+			matrix.Y0 = dst.Y - (matrix.Yy * src.Y);
+		}
+
+		public static Rectangle FromLTRB(double left, double top, double right, double bottom)
+		{
+			return new Rectangle(Math.Min(left, right), Math.Min(top, bottom),
+			                     Math.Abs(right - left), Math.Abs(bottom - top));
+		}
+
+		public static PointD GetCenter(this Cairo.Rectangle rect)
+		{
+			return new PointD(rect.X + rect.Width / 2, rect.Y + rect.Height / 2);
+		}
+
+		/// <summary>
+		/// Computes and returns the Union (largest possible combination) of two Rectangles.
+		/// The two given Rectangles do not need to intersect.
+		/// 
+		/// Another way to understand this function is that it computes and returns the
+		/// smallest possible Rectangle that encompasses both given Rectangles.
+		/// 
+		/// This function works as is intuitively expected with neither, either, or both given Rectangles being null.
+		/// </summary>
+		/// <param name="r1">The first given Rectangle.</param>
+		/// <param name="r2">The second given Rectangle.</param>
+		/// <returns></returns>
+		public static Rectangle? UnionRectangles(this Rectangle? r1, Rectangle? r2)
+		{
+			if (r1 == null)
+			{
+				//r2 is the only given Rectangle that could still have a value, and if it's null, return that anyways.
+				return r2;
+			}
+			else if (r2 == null)
+			{
+				//Only r1 has a value.
+				return r1;
+			}
+			else
+			{
+				//Both r1 and r2 have values.
+
+				//Calculate the left-most and top-most values.
+				double minX = Math.Min(r1.Value.X, r2.Value.X);
+				double minY = Math.Min(r1.Value.Y, r2.Value.Y);
+
+				//Calculate the right-most and bottom-most values and subtract the left-most and top-most values from them to get the width and height.
+				return new Rectangle(minX, minY,
+					Math.Max(r1.Value.X + r1.Value.Width, r2.Value.X + r2.Value.Width) - minX,
+					Math.Max(r1.Value.Y + r1.Value.Height, r2.Value.Y + r2.Value.Height) - minY);
+			}
+		}
+
+        public static Pattern ToTiledPattern (this Surface surface)
+        {
+            var pattern = new Cairo.SurfacePattern (surface);
+            pattern.Extend = Extend.Repeat;
+
+            return pattern;
+        }
 	}
 }
