@@ -31,6 +31,10 @@ namespace Pinta.Core
 {
 	public static class GtkExtensions
 	{
+		public const int MouseLeftButton = 1;
+		public const int MouseMiddleButton = 2;
+		public const int MouseRightButton = 3;
+
 		public static void AddWidgetItem (this Toolbar tb, Widget w)
 		{
 			w.Show ();
@@ -133,6 +137,11 @@ namespace Pinta.Core
 			return item;
 		}
 
+        public static void Toggle (this Gtk.ToggleToolButton button)
+        {
+            button.Active = !button.Active;
+        }
+
 		/// <summary>
 		/// Initialize an image preview widget for the dialog
 		/// </summary>
@@ -148,51 +157,85 @@ namespace Pinta.Core
 		/// <summary>
 		/// Update the image preview widget of a FileChooserDialog
 		/// </summary>
-		/// <param name="sender"></param>
 		private static void OnUpdateImagePreview (object sender, EventArgs e)
 		{
 			FileChooserDialog dialog = (FileChooserDialog)sender;
 			Image preview = (Image)dialog.PreviewWidget;
 
-            if (preview.Pixbuf != null)
-            {
-                preview.Pixbuf.Dispose ();
-            }
-
-			try
-			{
-                Gdk.Pixbuf pixbuf = null;
-                int imageWidth, imageHeight;
-                string filename = dialog.PreviewFilename;
-
-                var imageInfo = Gdk.Pixbuf.GetFileInfo (filename, out imageWidth, out imageHeight);
-
-                if (imageInfo == null)
-                {
-                    dialog.PreviewWidgetActive = false;
-                    return;
-                }
-
-                // scale down images that are too large, but don't scale up small images
-                if (imageWidth > MaxPreviewWidth || imageHeight > MaxPreviewHeight)
-                {
-                    pixbuf = new Gdk.Pixbuf (filename, MaxPreviewWidth, MaxPreviewHeight, true);
-                }
-                else
-                {
-                    pixbuf = new Gdk.Pixbuf (filename);
-                }
-
-                // add padding so that small images don't cause the dialog to shrink
-                preview.Xpad = (MaxPreviewWidth - pixbuf.Width) / 2;
-                preview.Pixbuf = pixbuf;
-                dialog.PreviewWidgetActive = true;
+			if (preview.Pixbuf != null) {
+				preview.Pixbuf.Dispose ();
 			}
-			catch (GLib.GException)
-			{
+
+			try {
+				Gdk.Pixbuf pixbuf = null;
+				string filename = dialog.PreviewFilename;
+
+				IImageImporter importer = PintaCore.System.ImageFormats.GetImporterByFile (filename);
+
+				if (importer != null) {
+					pixbuf = importer.LoadThumbnail (filename, MaxPreviewWidth, MaxPreviewHeight, dialog);
+				}
+
+				if (pixbuf == null) {
+					dialog.PreviewWidgetActive = false;
+					return;
+				}
+
+				// Resize the thumbnail in case the importer didn't.
+				if (pixbuf.Width > MaxPreviewWidth || pixbuf.Width > MaxPreviewHeight) {
+					double ratio = Math.Min ((double)MaxPreviewWidth / pixbuf.Width,
+					                         (double)MaxPreviewHeight / pixbuf.Height);
+					var old_pixbuf = pixbuf;
+					pixbuf = pixbuf.ScaleSimple ((int)(ratio * pixbuf.Width),
+					                             (int)(ratio * pixbuf.Height),
+					                             Gdk.InterpType.Bilinear);
+					old_pixbuf.Dispose ();
+				}
+
+				// add padding so that small images don't cause the dialog to shrink
+				preview.Xpad = (MaxPreviewWidth - pixbuf.Width) / 2;
+				preview.Pixbuf = pixbuf;
+				dialog.PreviewWidgetActive = true;
+			} catch (GLib.GException) {
 				// if the image preview failed, don't show the preview widget
 				dialog.PreviewWidgetActive = false;
 			}
 		}
+
+        public static int GetItemCount (this ComboBox combo)
+        {
+            return (combo.Model as ListStore).IterNChildren ();
+        }
+
+        public static int FindValue<T> (this ComboBox combo, T value)
+        {
+            for (var i = 0; i < combo.GetItemCount (); i++)
+                if (combo.GetValueAt<T> (i).Equals (value))
+                    return i;
+
+            return -1;
+        }
+
+        public static T GetValueAt<T> (this ComboBox combo, int index)
+        {
+            TreeIter iter;
+            
+            // Set the tree iter to the correct row
+            (combo.Model as ListStore).IterNthChild (out iter, index);
+
+            // Retrieve the value of the first column at that row
+            return (T)combo.Model.GetValue (iter, 0);
+        }
+
+        public static void SetValueAt (this ComboBox combo, int index, object value)
+        {
+            TreeIter iter;
+
+            // Set the tree iter to the correct row
+            (combo.Model as ListStore).IterNthChild (out iter, index);
+
+            // Set the value of the first column at that row
+            combo.Model.SetValue (iter, 0, value);
+        }
 	}
 }
