@@ -32,7 +32,10 @@ namespace Pinta.Core
 	public class SelectionHistoryItem : BaseHistoryItem
 	{
 		private DocumentSelection old_selection;
+		private DocumentSelection old_previous_selection;
+
 		private bool show_selection;
+		private bool hide_tool_layer;
 
 		public override bool CausesDirty { get { return false; } }
 
@@ -52,27 +55,40 @@ namespace Pinta.Core
 
 		public override void Dispose ()
 		{
-			old_selection.DisposeSelection();
+            old_selection.Dispose ();
+            old_previous_selection.Dispose ();
 		}
 
 		private void Swap ()
 		{
-			DocumentSelection swap_selection = PintaCore.Workspace.ActiveDocument.Selection;
+			var doc = PintaCore.Workspace.ActiveDocument;
+			DocumentSelection swap_selection = doc.Selection;
 			bool swap_show = PintaCore.Layers.ShowSelection;
+			bool swap_hide_tool_layer = doc.ToolLayer.Hidden;
 
-			PintaCore.Workspace.ActiveDocument.Selection = old_selection;
+			doc.Selection = old_selection;
 			PintaCore.Layers.ShowSelection = show_selection;
+			doc.ToolLayer.Hidden = hide_tool_layer;
 
 			old_selection = swap_selection;
 			show_selection = swap_show;
-			
+			hide_tool_layer = swap_hide_tool_layer;
+
+            swap_selection = old_previous_selection;
+            old_previous_selection = doc.PreviousSelection;
+            doc.PreviousSelection = swap_selection;
+
+			PintaCore.Workspace.CallSelectionChanged(this, EventArgs.Empty);
 			PintaCore.Workspace.Invalidate ();
 		}
 		
 		public void TakeSnapshot ()
 		{
-			old_selection = PintaCore.Workspace.ActiveDocument.Selection.Clone ();
+			var doc = PintaCore.Workspace.ActiveDocument;
+			old_selection = doc.Selection.Clone ();
+            old_previous_selection = doc.PreviousSelection.Clone ();
 			show_selection = PintaCore.Layers.ShowSelection;
+			hide_tool_layer = doc.ToolLayer.Hidden;
 		}
 	}
 }
