@@ -25,11 +25,25 @@
 // THE SOFTWARE.
 
 using System;
+using Mono.Addins;
 
 namespace Pinta.Core
 {
+	[TypeExtensionPoint]
 	public interface IImageExporter
 	{
-		void Export (Document document, string fileName);
+		/// <summary>
+		/// Exports a document to a file.
+		/// </summary>
+		/// <param name='document'>
+		/// The document to be saved.
+		/// </param>
+		/// <param name='fileName'>
+		/// File name to save to.
+		/// </param>
+		/// <param name='parent'>
+		/// Window to be used as a parent for any dialogs that are shown.
+		/// </param>
+		void Export (Document document, string fileName, Gtk.Window parent);
 	}
 }
