@@ -26,6 +26,8 @@
 
 using System;
 using Gtk;
+using System.Collections.Generic;
+using ClipperLibrary;
 
 namespace Pinta.Core
 {
@@ -40,6 +42,7 @@ namespace Pinta.Core
 		public LayerActions Layers { get; private set; }
 		public AdjustmentsActions Adjustments { get; private set; }
 		public EffectsActions Effects { get; private set; }
+		public AddinActions Addins { get; private set; }
 		public WindowActions Window { get; private set; }
 		public HelpActions Help { get; private set; }
 		
@@ -54,6 +57,7 @@ namespace Pinta.Core
 			Layers = new LayerActions ();
 			Adjustments = new AdjustmentsActions ();
 			Effects = new EffectsActions ();
+			Addins = new AddinActions ();
 			Window = new WindowActions ();
 			Help = new HelpActions ();
 		}
@@ -94,13 +98,18 @@ namespace Pinta.Core
 			eff.Submenu = new Menu ();
 			Effects.CreateMainMenu ((Menu)eff.Submenu);
 
+			// Add-ins menu
+			ImageMenuItem addins = (ImageMenuItem)menu.Children[7];
+			addins.Submenu = new Menu ();
+			Addins.CreateMainMenu ((Menu)addins.Submenu);
+
 			// Window menu
-			ImageMenuItem window = (ImageMenuItem)menu.Children[7];
+			ImageMenuItem window = (ImageMenuItem)menu.Children[8];
 			window.Submenu = new Menu ();
 			Window.CreateMainMenu ((Menu)window.Submenu);
 			
 			//Help menu
-			ImageMenuItem help = (ImageMenuItem)menu.Children[8];
+			ImageMenuItem help = (ImageMenuItem)menu.Children[9];
 			help.Submenu = new Menu ();
 			Help.CreateMainMenu ((Menu)help.Submenu);
 		}
@@ -110,7 +119,6 @@ namespace Pinta.Core
 			toolbar.AppendItem (File.New.CreateToolBarItem ());
 			toolbar.AppendItem (File.Open.CreateToolBarItem ());
 			toolbar.AppendItem (File.Save.CreateToolBarItem ());
-			//toolbar.AppendItem (File.Print.CreateToolBarItem ());
 			toolbar.AppendItem (new SeparatorToolItem ());
 
 			// Cut/Copy/Paste comes before Undo/Redo on Windows
@@ -135,6 +143,7 @@ namespace Pinta.Core
 			toolbar.AppendItem (Edit.Deselect.CreateToolBarItem ());
 			View.CreateToolBar (toolbar);
 
+
 			toolbar.AppendItem (new SeparatorToolItem ());
 			toolbar.AppendItem (new ToolBarImage ("StatusBar.CursorXY.png"));
 
@@ -146,6 +155,64 @@ namespace Pinta.Core
 				Gdk.Point pt = PintaCore.Chrome.LastCanvasCursorPoint;
 				cursor.Text = string.Format ("  {0}, {1}", pt.X, pt.Y);
 			};
+
+
+			toolbar.AppendItem(new SeparatorToolItem());
+			toolbar.AppendItem(new ToolBarImage("Tools.RectangleSelect.png"));
+
+			ToolBarLabel SelectionSize = new ToolBarLabel("  0, 0");
+
+			toolbar.AppendItem(SelectionSize);
+
+			PintaCore.Workspace.SelectionChanged += delegate
+			{
+				double minX = double.MaxValue;
+				double minY = double.MaxValue;
+				double maxX = double.MinValue;
+				double maxY = double.MinValue;
+
+				//Calculate the minimum rectangular bounds that surround the current selection.
+				foreach (List<IntPoint> li in PintaCore.Workspace.ActiveDocument.Selection.SelectionPolygons)
+				{
+					foreach (IntPoint ip in li)
+					{
+						if (minX > ip.X)
+						{
+							minX = ip.X;
+						}
+
+						if (minY > ip.Y)
+						{
+							minY = ip.Y;
+						}
+
+						if (maxX < ip.X)
+						{
+							maxX = ip.X;
+						}
+
+						if (maxY < ip.Y)
+						{
+							maxY = ip.Y;
+						}
+					}
+				}
+
+				double xDiff = maxX - minX;
+				double yDiff = maxY - minY;
+
+				if (double.IsNegativeInfinity(xDiff))
+				{
+					xDiff = 0d;
+				}
+
+				if (double.IsNegativeInfinity(yDiff))
+				{
+					yDiff = 0d;
+				}
+
+				SelectionSize.Text = string.Format("  {0}, {1}", xDiff, yDiff);
+			};
 		}
 		
 		public void RegisterHandlers ()
