@@ -50,6 +50,7 @@ namespace Pinta.Core
 		
 		Cairo.ImageSurface live_preview_surface;
 		Gdk.Rectangle render_bounds;
+		SimpleHistoryItem history_item;
 		
 		AsyncEffectRenderer renderer;
 		
@@ -62,6 +63,7 @@ namespace Pinta.Core
 		
 		public bool IsEnabled { get { return live_preview_enabled; } }
 		public Cairo.ImageSurface LivePreviewSurface { get { return live_preview_surface; } }
+		public Gdk.Rectangle RenderBounds { get { return render_bounds; } }
 		
 		public event EventHandler<LivePreviewStartedEventArgs> Started;
 		public event EventHandler<LivePreviewRenderUpdatedEventArgs> RenderUpdated;
@@ -75,29 +77,31 @@ namespace Pinta.Core
 			// Create live preview surface.
 			// Start rendering.
 			// Listen for changes to effectConfiguration object, and restart render if needed.
-			
+
 			live_preview_enabled = true;
 			apply_live_preview_flag = false;
 			cancel_live_preview_flag = false;
 			
 			layer = PintaCore.Layers.CurrentLayer;
 			this.effect = effect;
-			
-			// Handle selection path.
-			PintaCore.Tools.Commit ();
-			selection_path = (PintaCore.Layers.ShowSelection) ? PintaCore.Workspace.ActiveDocument.Selection.SelectionPath : null;
-			render_bounds = selection_path.GetBounds ();
-			render_bounds = PintaCore.Workspace.ClampToImageSize (render_bounds);			
-									
+
 			//TODO Use the current tool layer instead.
 			live_preview_surface = new Cairo.ImageSurface (Cairo.Format.Argb32,
 			                                  PintaCore.Workspace.ImageSize.Width,
 			                                  PintaCore.Workspace.ImageSize.Height);
+
+			// Handle selection path.
+			PintaCore.Tools.Commit ();
+			selection_path = (PintaCore.Layers.ShowSelection) ? PintaCore.Workspace.ActiveDocument.Selection.SelectionPath : null;
+			render_bounds = (selection_path != null) ? selection_path.GetBounds () : live_preview_surface.GetBounds ();
+			render_bounds = PintaCore.Workspace.ClampToImageSize (render_bounds);	
+
+			history_item = new SimpleHistoryItem (effect.Icon, effect.Name);
+			history_item.TakeSnapshotOfLayer (PintaCore.Layers.CurrentLayerIndex);	
 			
 			// Paint the pre-effect layer surface into into the working surface.
 			using (var ctx = new Cairo.Context (live_preview_surface)) {
-				ctx.SetSourceSurface (layer.Surface, (int) layer.Offset.X, (int) layer.Offset.Y);
-				ctx.Paint ();
+				layer.Draw(ctx, layer.Surface, 1);
 			}
 			
 			if (effect.EffectData != null)
@@ -150,15 +154,14 @@ namespace Pinta.Core
 				ctx.Rectangle (0, 0, imageSize.Width, imageSize.Height);
 				ctx.AppendPath (selection_path);
 				ctx.Clip ();
-				ctx.SetSourceSurface (layer.Surface, (int)layer.Offset.X, (int)layer.Offset.Y);
-				ctx.PaintWithAlpha (opacity);
+				layer.Draw(ctx, layer.Surface, opacity);
 				ctx.ResetClip ();
 				
 				// Paint area inside the selection path, with the post-effect image.
 				ctx.AppendPath (selection_path);
 				ctx.Clip ();
 				
-				ctx.SetSourceSurface (live_preview_surface, (int)layer.Offset.X, (int)layer.Offset.Y);
+				layer.Draw(ctx, live_preview_surface, opacity);
 				ctx.PaintWithAlpha (opacity);
 				
 				ctx.AppendPath (selection_path);
@@ -166,8 +169,7 @@ namespace Pinta.Core
 				ctx.Clip ();			
 			} else {
 				
-				ctx.SetSourceSurface (live_preview_surface, (int)layer.Offset.X, (int)layer.Offset.Y);
-				ctx.PaintWithAlpha (opacity);				
+				layer.Draw(ctx, live_preview_surface, opacity);
 			}
 			ctx.Restore ();
 		}
@@ -200,8 +202,7 @@ namespace Pinta.Core
 			live_preview_enabled = false;
 			
 			if (live_preview_surface != null) {
-				var disposable = live_preview_surface as IDisposable;
-					disposable.Dispose ();
+				(live_preview_surface as IDisposable).Dispose ();
 			}
 			
 			PintaCore.Workspace.Invalidate ();
@@ -235,24 +236,19 @@ namespace Pinta.Core
 		{
 			Debug.WriteLine ("LivePreviewManager.HandleApply()");
 
-			var item = new SimpleHistoryItem (effect.Icon, effect.Name);
-			item.TakeSnapshotOfLayer (PintaCore.Layers.CurrentLayerIndex);			
-			
 			using (var ctx = new Cairo.Context (layer.Surface)) {
 				
 				ctx.Save ();
-				ctx.AppendPath (PintaCore.Workspace.ActiveDocument.Selection.SelectionPath);
-				ctx.FillRule = Cairo.FillRule.EvenOdd;
-				ctx.Clip ();				
+				PintaCore.Workspace.ActiveDocument.Selection.Clip (ctx);
 			
 				ctx.Operator = Cairo.Operator.Source;
 				
-				ctx.SetSourceSurface (live_preview_surface, (int)layer.Offset.X, (int)layer.Offset.Y);
-				ctx.Paint ();
+				layer.Draw(ctx, live_preview_surface, 1);
 				ctx.Restore ();
 			}
 			
-			PintaCore.History.PushNewItem (item);
+			PintaCore.History.PushNewItem (history_item);
+			history_item = null;
 			
 			FireLivePreviewEndedEvent(RenderStatus.Completed, null);
 			
@@ -281,6 +277,11 @@ namespace Pinta.Core
 				renderer.Dispose ();
 				renderer = null;
 			}
+
+			if (history_item != null) {
+				history_item.Dispose ();
+				history_item = null;
+			}
 			
 			// Hide progress dialog and clean up events.
 			var dialog = PintaCore.Chrome.ProgressDialog;
