@@ -37,10 +37,18 @@ namespace Pinta.Core
 	{
 		private int active_document_index = -1;
 		private int new_file_name = 1;
+
+		public event EventHandler SelectionChanged;
 		
 		public WorkspaceManager ()
 		{
 			OpenDocuments = new List<Document> ();
+            SelectionHandler = new SelectionModeHandler ();
+		}
+
+		public void CallSelectionChanged(object sender, EventArgs e)
+		{
+			SelectionChanged(sender, e);
 		}
 
 		public int ActiveDocumentIndex {
@@ -58,6 +66,8 @@ namespace Pinta.Core
 			}
 		}
 
+        public SelectionModeHandler SelectionHandler { get; private set; }
+
 		public DocumentWorkspace ActiveWorkspace {
 			get {
 				if (HasOpenDocuments)
@@ -147,7 +157,7 @@ namespace Pinta.Core
 			ActiveWorkspace.Invalidate (rect);
 		}
 
-		public Document NewDocument (Gdk.Size imageSize, bool transparent)
+		public Document NewDocument (Gdk.Size imageSize, Color backgroundColor)
 		{
 			Document doc = CreateAndActivateDocument (null, imageSize);
 			doc.Workspace.CanvasSize = imageSize;
@@ -155,9 +165,9 @@ namespace Pinta.Core
 			// Start with an empty white layer
 			Layer background = doc.AddNewLayer (Catalog.GetString ("Background"));
 
-			if (!transparent) {
+			if (backgroundColor.A != 0) {
 				using (Cairo.Context g = new Cairo.Context (background.Surface)) {
-					g.SetSourceRGB (1, 1, 1);
+					g.SetSourceColor (backgroundColor);
 					g.Paint ();
 				}
 			}
@@ -175,10 +185,16 @@ namespace Pinta.Core
 		{
 			bool fileOpened = false;
 
+			if (parent == null)
+				parent = PintaCore.Chrome.MainWindow;
+
 			try {
 				// Open the image and add it to the layers
 				IImageImporter importer = PintaCore.System.ImageFormats.GetImporterByFile (file);
-				importer.Import (file);
+				if (importer == null)
+					throw new FormatException( Catalog.GetString ("Unsupported file format"));
+
+				importer.Import (file, parent);
 
 				PintaCore.Workspace.ActiveDocument.PathAndFileName = file;
 				PintaCore.Workspace.ActiveWorkspace.History.PushNewItem (new BaseHistoryItem (Stock.Open, Catalog.GetString ("Open Image")));
@@ -188,10 +204,10 @@ namespace Pinta.Core
 				PintaCore.Workspace.Invalidate ();
 
 				fileOpened = true;
-			} catch (UnauthorizedAccessException) {
-				ShowOpenFileErrorDialog (parent, file, Catalog.GetString ("Permission denied"));
+			} catch (UnauthorizedAccessException e) {
+				ShowOpenFileErrorDialog (parent, file, Catalog.GetString ("Permission denied"), e.ToString ());
 			} catch (Exception e) {
-				ShowOpenFileErrorDialog (parent, file, e.Message);
+				ShowOpenFileErrorDialog (parent, file, e.Message, e.ToString ());
 			}
 
 			return fileOpened;
@@ -206,12 +222,37 @@ namespace Pinta.Core
 		{
 			ActiveDocument.ResizeCanvas (width, height, anchor, compoundAction);
 		}
-		
+
+		/// <summary>
+		/// Converts a point from the active documents
+		/// window coordinates to canvas coordinates
+		/// </summary>
+		/// <param name='x'>
+		/// The X coordinate of the window point
+		/// </param>
+		/// <param name='y'>
+		/// The Y coordinate of the window point
+		/// </param>
 		public Cairo.PointD WindowPointToCanvas (double x, double y)
 		{
 			return ActiveWorkspace.WindowPointToCanvas (x, y);
 		}
 
+		/// <summary>
+		/// Converts a point from the active documents
+		/// canvas coordinates to window coordinates
+		/// </summary>
+		/// <param name='x'>
+		/// The X coordinate of the canvas point
+		/// </param>
+		/// <param name='y'>
+		/// The Y coordinate of the canvas point
+		/// </param>
+		public Cairo.PointD CanvasPointToWindow (double x, double y)
+		{
+			return ActiveWorkspace.CanvasPointToWindow (x, y);
+		}
+
 		public Gdk.Rectangle ClampToImageSize (Gdk.Rectangle r)
 		{
 			return ActiveDocument.ClampToImageSize (r);
@@ -302,19 +343,12 @@ namespace Pinta.Core
 		}
 		#endregion
 
-		private void ShowOpenFileErrorDialog (Window parent, string filename, string primaryText)
+		private void ShowOpenFileErrorDialog (Window parent, string filename, string primaryText, string details)
 		{
-			if (parent == null)
-				parent = PintaCore.Chrome.MainWindow;
-
 			string markup = "<span weight=\"bold\" size=\"larger\">{0}</span>\n\n{1}";
 			string secondaryText = string.Format (Catalog.GetString ("Could not open file: {0}"), filename);
 			string message = string.Format (markup, primaryText, secondaryText);
-
-			var md = new MessageDialog (parent, DialogFlags.Modal | DialogFlags.DestroyWithParent,
-			                            MessageType.Error, ButtonsType.Ok, message);
-			md.Run ();
-			md.Destroy ();
+			PintaCore.Chrome.ShowErrorDialog(parent, message, details);
 		}
 
 		#region Public Events
