@@ -13,7 +13,6 @@ using Pinta.Core;
 
 namespace Pinta.Effects
 {
-	//[System.ComponentModel.Composition (typeof (BaseEffect))]
 	public class SepiaEffect : BaseEffect
 	{
 		UnaryPixelOp desat = new UnaryPixelOps.Desaturate ();
@@ -23,13 +22,9 @@ namespace Pinta.Effects
 			get { return "Menu.Adjustments.Sepia.png"; }
 		}
 
-		public override string Text {
+		public override string Name {
 			get { return Mono.Unix.Catalog.GetString ("Sepia"); }
 		}
-		
-		public override EffectAdjustment EffectOrAdjustment {
-			get { return EffectAdjustment.Adjustment; }
-		}
 
 		public override Gdk.Key AdjustmentMenuKey {
 			get { return Gdk.Key.E; }
@@ -46,7 +41,7 @@ namespace Pinta.Effects
 				ColorBgra.White);
 		}
 
-		public override void RenderEffect (ImageSurface src, ImageSurface dest, Gdk.Rectangle[] rois)
+		public override void Render (ImageSurface src, ImageSurface dest, Gdk.Rectangle[] rois)
 		{
 			desat.Apply (dest, src, rois);
 			level.Apply (dest, dest, rois);
