@@ -55,8 +55,7 @@ namespace Pinta.Effects
 			float hw = dst.Width / 2f;
 			float hh = dst.Height / 2f;
 			float maxrad = Math.Min (hw, hh);
-			float maxrad2 = maxrad * maxrad;
-			float amt = Data.Amount / 100f;
+			float amt = bulge / 100f;
 			
 			hh = hh + (float)Data.Offset.Y * hh;
 			hw = hw + (float)Data.Offset.X * hw;
@@ -99,9 +98,10 @@ namespace Pinta.Effects
 
 		public class BulgeData : EffectData
 		{
-			[MinimumValue(-200), MaximumValue(100)]
+			[Caption ("Amount"), MinimumValue(-200), MaximumValue(100)]
 			public int Amount = 45;
 
+			[Caption ("Offset")]
 			public Cairo.PointD Offset = new Cairo.PointD (0.0, 0.0);
 
 			[Skip]
