@@ -50,7 +50,6 @@ namespace Pinta.Effects
 		{
 			double pointStep = 2 * Math.PI / (double)fragments;
 			double rotationRadians = ((rotationAngle - 90.0) * Math.PI) / 180.0;
-			double offsetAngle = pointStep;
 
 			Gdk.Point[] pointOffsets = new Gdk.Point[fragments];
 
@@ -109,12 +108,13 @@ namespace Pinta.Effects
 
 		public class FragmentData : EffectData
 		{
-			[MinimumValue (2), MaximumValue (50)]
+			[Caption ("Fragments"), MinimumValue (2), MaximumValue (50)]
 			public int Fragments = 4;
 
-			[MinimumValue (0), MaximumValue (100)]
+			[Caption ("Distance"), MinimumValue (0), MaximumValue (100)]
 			public int Distance = 8;
 
+			[Caption ("Rotation")]
 			public double Rotation = 0;
 		}
 	}
