@@ -93,8 +93,6 @@ namespace Pinta.Effects
 			long* rSums = (long*)p;
 			p += wlen * sizeof (long);
 
-			ulong arraysLength = (ulong)(sizeof (long) * wlen);
-
 			// Cache these for a massive performance boost
 			int src_width = src.Width;
 			int src_height = src.Height;
@@ -259,7 +257,7 @@ namespace Pinta.Effects
 
 		public class GaussianBlurData : EffectData
 		{
-			[MinimumValue (0), MaximumValue (200)]
+			[Caption ("Radius"), MinimumValue (0), MaximumValue (200)]
 			public int Radius = 2;
 			
 			[Skip]
