@@ -78,9 +78,6 @@ namespace Pinta.Effects
 			int w = dst.Width;
 			int h = dst.Height;
 			
-			double wDiv2 = (double)w / 2;
-			double hDiv2 = (double)h / 2;
-			
 			double invH = 1.0 / h;
 			double zoom = 1 + zoomFactor * Data.Zoom;
 			double invZoom = 1.0 / zoom;
@@ -154,18 +151,20 @@ namespace Pinta.Effects
 
 		public class MandelbrotFractalData : EffectData
 		{
-			[MinimumValue(1), MaximumValue(10)]
+			[Caption ("Factor"), MinimumValue(1), MaximumValue(10)]
 			public int Factor = 1;
 
-			[MinimumValue(1), MaximumValue(5)]
+			[Caption ("Quality"), MinimumValue(1), MaximumValue(5)]
 			public int Quality = 2;
 
 			//TODO double
-			[MinimumValue(0), MaximumValue(100)]
+			[Caption ("Zoom"), MinimumValue(0), MaximumValue(100)]
 			public int Zoom = 10;
 
+			[Caption ("Angle")]
 			public double Angle = 0;
 
+			[Caption ("Invert Colors")]
 			public bool InvertColors = false;
 			
 		}
