@@ -40,6 +40,11 @@ namespace Pinta.Gui.Widgets
 	{
 		private TreeView tree;
 		private TreeStore store;
+
+		// For the active layer, we also draw the selection layer on top of it,
+		// so we can't directly use that layer's surface.
+		private Cairo.ImageSurface active_layer_surface;
+		private CanvasRenderer canvas_renderer = new CanvasRenderer (false);
 				
 		private const int store_index_thumbnail = 0;
 		private const int store_index_name = 1;
@@ -117,16 +122,16 @@ namespace Pinta.Gui.Widgets
 
 			ShowAll ();
 		}
-		
-		private Layer GetSelectedLayerInTreeView ()
+
+		private UserLayer GetSelectedLayerInTreeView()
 		{
-			Layer layer = null;
+			UserLayer layer = null;
 			TreeIter iter;
 			
 			var paths = tree.Selection.GetSelectedRows ();
 				
 			if (paths != null && paths.Length > 0 && store.GetIter (out iter, paths[0])) {
-				layer = store.GetValue (iter, store_index_layer) as Layer;
+				layer = store.GetValue(iter, store_index_layer) as UserLayer;
 			}
 			
 			return layer;
@@ -151,8 +156,8 @@ namespace Pinta.Gui.Widgets
 			if (store.GetIter (out iter, new TreePath (args.Path))) {
 				bool b = (bool) store.GetValue (iter, store_index_visibility);				
 				store.SetValue(iter, store_index_visibility, !b);
-				
-				var layer = (Layer) store.GetValue (iter, store_index_layer);
+
+				var layer = (UserLayer)store.GetValue(iter, store_index_layer);
 				SetLayerVisibility (layer, !b);
 			}
 		}
@@ -194,16 +199,39 @@ namespace Pinta.Gui.Widgets
 		{
 			store.Clear ();
 
+			if (active_layer_surface != null) {
+				(active_layer_surface as IDisposable).Dispose ();
+				active_layer_surface = null;
+			}
+
 			if (!PintaCore.Workspace.HasOpenDocuments)
 				return;
+
+			var doc = PintaCore.Workspace.ActiveDocument;
 				
-			foreach (var layer in (PintaCore.Workspace.ActiveDocument.Layers as IEnumerable<Layer>).Reverse ())
-				store.AppendValues (layer.Surface, layer.Name, !layer.Hidden, layer);
+			foreach (var layer in (doc.UserLayers as IEnumerable<Layer>).Reverse ()) {
+				var surf = layer.Surface;
+
+				// Draw the selection layer on top of the active layer.
+				if (layer == doc.CurrentUserLayer && doc.ShowSelectionLayer) {
+					active_layer_surface = new Cairo.ImageSurface (Cairo.Format.Argb32, thumbnail_width,
+					                                               thumbnail_height);
+					canvas_renderer.Initialize (doc.ImageSize,
+					                            new Gdk.Size (thumbnail_width, thumbnail_height));
+
+					var layers = new List<Layer> { layer, doc.SelectionLayer };
+					canvas_renderer.Render (layers, active_layer_surface, Gdk.Point.Zero);
+
+					surf = active_layer_surface;
+				}
+
+				store.AppendValues (surf, layer.Name, !layer.Hidden, layer);
+			}
 						
 			SelectLayerInTreeView (PintaCore.Layers.Count - PintaCore.Layers.CurrentLayerIndex - 1);
-		}		
-		
-		private void SetLayerVisibility (Layer layer, bool visibility)
+		}
+
+		private void SetLayerVisibility(UserLayer layer, bool visibility)
 		{
 			if (layer != null)
 				layer.Hidden = !visibility;
@@ -221,7 +249,7 @@ namespace Pinta.Gui.Widgets
 			PintaCore.History.PushNewItem (historyItem);
 			
 			//TODO Call this automatically when the layer visibility changes.
-			PintaCore.Workspace.Invalidate ();			
+			PintaCore.Workspace.Invalidate ();
 		}
 	}
 }
