@@ -45,11 +45,30 @@ namespace Pinta.Tools
 			get { return "Tools.CloneStamp.png"; }
 		}
 		public override string StatusBarText { get { return Catalog.GetString ("Ctrl-left click to set origin, left click to paint."); } }
-		public override Gdk.Cursor DefaultCursor { get { return new Gdk.Cursor (PintaCore.Chrome.Canvas.Display, PintaCore.Resources.GetIcon ("Cursor.CloneStamp.png"), 6, 11); } }
+
+		public override Gdk.Cursor DefaultCursor {
+			get {
+				int iconOffsetX, iconOffsetY;
+				var icon = CreateIconWithShape ("Cursor.CloneStamp.png",
+				                                CursorShape.Ellipse, BrushWidth, 16, 26,
+				                                out iconOffsetX, out iconOffsetY);
+				return new Gdk.Cursor (PintaCore.Chrome.Canvas.Display, icon, iconOffsetX, iconOffsetY);
+			}
+		}
+		public override bool CursorChangesOnZoom { get { return true; } }
+
 		public override Gdk.Key ShortcutKey { get { return Gdk.Key.L; } }
 		public override int Priority { get { return 33; } }
 		protected override bool ShowAntialiasingButton { get { return true; } }
 
+		protected override void OnBuildToolBar(Gtk.Toolbar tb)
+		{
+			base.OnBuildToolBar(tb);
+
+			// Change the cursor when the BrushWidth is changed.
+			brush_width.ComboBox.Changed += (sender, e) => SetCursor (DefaultCursor);
+		}
+
 		protected override void OnMouseDown (Gtk.DrawingArea canvas, Gtk.ButtonPressEventArgs args, Cairo.PointD point)
 		{
 			Document doc = PintaCore.Workspace.ActiveDocument;
@@ -72,7 +91,7 @@ namespace Pinta.Tools
 				doc.ToolLayer.Hidden = false;
 
 				surface_modified = false;
-				undo_surface = doc.CurrentLayer.Surface.Clone ();
+				undo_surface = doc.CurrentUserLayer.Surface.Clone ();
 			} else {
 				origin = point.ToGdkPoint ();
 			}
@@ -99,7 +118,7 @@ namespace Pinta.Tools
 				g.MoveTo (last_point.X, last_point.Y);
 				g.LineTo (x, y);
 
-				g.SetSource (doc.CurrentLayer.Surface, offset.X, offset.Y);
+				g.SetSource (doc.CurrentUserLayer.Surface, offset.X, offset.Y);
 				g.LineWidth = BrushWidth;
 				g.LineCap = Cairo.LineCap.Round;
 
@@ -119,7 +138,7 @@ namespace Pinta.Tools
 
 			painting = false;
 
-			using (Cairo.Context g = new Cairo.Context (doc.CurrentLayer.Surface)) {
+			using (Cairo.Context g = new Cairo.Context (doc.CurrentUserLayer.Surface)) {
 				g.SetSource (doc.ToolLayer.Surface);
 				g.Paint ();
 			}
@@ -140,7 +159,7 @@ namespace Pinta.Tools
 			//note that this WONT work if user presses control key and THEN selects the tool!
 			if (args.Event.Key == Key.Control_L || args.Event.Key == Key.Control_R) {
 				Gdk.Pixbuf icon = PintaCore.Resources.GetIcon ("Cursor.CloneStampSetSource.png");
-				Gdk.Cursor setSourceCursor = new Gdk.Cursor(PintaCore.Chrome.Canvas.Display, icon, 6, 11);
+				Gdk.Cursor setSourceCursor = new Gdk.Cursor(PintaCore.Chrome.Canvas.Display, icon, 16, 26);
 				SetCursor(setSourceCursor);
 			}
 		}
@@ -152,7 +171,7 @@ namespace Pinta.Tools
 				SetCursor(DefaultCursor);
 		}
 
-		protected override void OnDeactivated ()
+		protected override void OnDeactivated(BaseTool newTool)
 		{
 			origin = new Point (int.MinValue, int.MinValue);
 		}
