@@ -40,10 +40,30 @@ namespace Pinta.Tools
 		{
 		}
 
+		protected override void OnBuildToolBar(Toolbar tb)
+		{
+			base.OnBuildToolBar(tb);
+
+			// Change the cursor when the BrushWidth is changed.
+			brush_width.ComboBox.Changed += (sender, e) => SetCursor (DefaultCursor);
+		}
+
 		#region Properties
 		public override string Name { get { return Catalog.GetString ("Eraser"); } }
 		public override string Icon { get { return "Tools.Eraser.png"; } }
 		public override string StatusBarText { get { return Catalog.GetString ("Left click to erase to transparent, right click to erase to secondary color. "); } }
+
+		public override Gdk.Cursor DefaultCursor {
+			get {
+				int iconOffsetX, iconOffsetY;
+				var icon = CreateIconWithShape ("Cursor.Eraser.png",
+				                                CursorShape.Ellipse, BrushWidth, 8, 22,
+				                                out iconOffsetX, out iconOffsetY);
+				return new Gdk.Cursor (PintaCore.Chrome.Canvas.Display, icon, iconOffsetX, iconOffsetY);
+			}
+		}
+		public override bool CursorChangesOnZoom { get { return true; } }
+
 		public override Gdk.Key ShortcutKey { get { return Gdk.Key.E; } }
 		public override int Priority { get { return 27; } }
 		#endregion
@@ -58,8 +78,6 @@ namespace Pinta.Tools
 				return;
 			}
 
-			DrawingArea drawingarea1 = (DrawingArea)o;
-			
 			int x = (int)point.X;
 			int y = (int)point.Y;
 			
@@ -69,7 +87,7 @@ namespace Pinta.Tools
 			if (doc.Workspace.PointInCanvas (point))
 				surface_modified = true;
 
-			ImageSurface surf = doc.CurrentLayer.Surface;
+			ImageSurface surf = doc.CurrentUserLayer.Surface;
 			
 			using (Context g = new Context (surf)) {
 				g.AppendPath (doc.Selection.SelectionPath);
@@ -85,7 +103,7 @@ namespace Pinta.Tools
 
 				// Right-click is erase to background color, left-click is transparent
 				if (mouse_button == 3)
-					g.Color = PintaCore.Palette.SecondaryColor;
+					g.SetSourceColor (PintaCore.Palette.SecondaryColor);
 				else
 					g.Operator = Operator.Clear;
 
@@ -101,7 +119,7 @@ namespace Pinta.Tools
 			if (doc.Workspace.IsPartiallyOffscreen (r)) {
 				doc.Workspace.Invalidate ();
 			} else {
-				doc.Workspace.Invalidate (r);
+				doc.Workspace.Invalidate (doc.ClampToImageSize (r));
 			}
 			
 			last_point = new Point (x, y);
