@@ -40,25 +40,54 @@ namespace Pinta.Tools
 		public override string Name { get { return Catalog.GetString ("Paintbrush"); } }
 		public override string Icon { get { return "Tools.Paintbrush.png"; } }
 		public override string StatusBarText { get { return Catalog.GetString ("Left click to draw with primary color, right click to draw with secondary color."); } }
+
+		public override Gdk.Cursor DefaultCursor {
+			get {
+				int iconOffsetX, iconOffsetY;
+				var icon = CreateIconWithShape ("Cursor.Paintbrush.png",
+				                                CursorShape.Ellipse, BrushWidth, 8, 24,
+				                                out iconOffsetX, out iconOffsetY);
+				return new Gdk.Cursor (PintaCore.Chrome.Canvas.Display, icon, iconOffsetX, iconOffsetY);
+			}
+		}
+		public override bool CursorChangesOnZoom { get { return true; } }
+
 		public override Gdk.Key ShortcutKey { get { return Gdk.Key.B; } }
 		public override int Priority { get { return 25; } }
-
-		public Color StrokeColor { get; private set; }
-		public Color FillColor { get; private set; }
-		public Point LastPoint { get; private set; }
-		public Cairo.Context Drawable { get; private set; }
-		public ImageSurface Surface { get; private set; }
 		#endregion
 
-		private PaintBrush default_brush;
-		private PaintBrush active_brush;
+		private BasePaintBrush default_brush;
+		private BasePaintBrush active_brush;
 		private ToolBarLabel brush_label;
 		private ToolBarComboBox brush_combo_box;
+		private Color stroke_color;
+        private Point last_point;
+
+		protected override void OnActivated ()
+		{
+			base.OnActivated ();
+
+			PintaCore.PaintBrushes.BrushAdded += HandleBrushAddedOrRemoved;
+			PintaCore.PaintBrushes.BrushRemoved += HandleBrushAddedOrRemoved;
+		}
+
+		protected override void OnDeactivated (BaseTool newTool)
+		{
+			base.OnDeactivated (newTool);
+
+			PintaCore.PaintBrushes.BrushAdded -= HandleBrushAddedOrRemoved;
+			PintaCore.PaintBrushes.BrushRemoved -= HandleBrushAddedOrRemoved;
+		}
 
 		protected override void OnBuildToolBar (Toolbar tb)
 		{
 			base.OnBuildToolBar (tb);
 
+			// Change the cursor when the BrushWidth is changed.
+			brush_width.ComboBox.Changed += (sender, e) => SetCursor (DefaultCursor);
+
+			tb.AppendItem (new Gtk.SeparatorToolItem ());
+
 			if (brush_label == null)
 				brush_label = new ToolBarLabel (string.Format (" {0}:  ", Catalog.GetString ("Type")));
 
@@ -67,23 +96,44 @@ namespace Pinta.Tools
 				brush_combo_box.ComboBox.Changed += (o, e) => {
 					Gtk.TreeIter iter;
 					if (brush_combo_box.ComboBox.GetActiveIter (out iter)) {
-						active_brush = (PaintBrush)brush_combo_box.Model.GetValue (iter, 1);
+						active_brush = (BasePaintBrush)brush_combo_box.Model.GetValue (iter, 1);
 					} else {
 						active_brush = default_brush;
 					}
 				};
-				foreach (var brush in PintaCore.PaintBrushes) {
-					if (default_brush == null)
-						default_brush = (PaintBrush)brush;
-					brush_combo_box.Model.AppendValues (brush.Name, brush);
-				}
-				brush_combo_box.ComboBox.Active = 0;
+
+				RebuildBrushComboBox ();
 			}
 
 			tb.AppendItem (brush_label);
 			tb.AppendItem (brush_combo_box);
 		}
 
+		/// <summary>
+		/// Rebuild the list of brushes when a brush is added or removed.
+		/// </summary>
+		private void HandleBrushAddedOrRemoved (object sender, BrushEventArgs e)
+		{
+			RebuildBrushComboBox ();
+		}
+
+		/// <summary>
+		/// Rebuild the list of brushes.
+		/// </summary>
+		private void RebuildBrushComboBox ()
+		{
+			brush_combo_box.Model.Clear ();
+			default_brush = null;
+
+			foreach (var brush in PintaCore.PaintBrushes) {
+				if (default_brush == null)
+					default_brush = (BasePaintBrush)brush;
+				brush_combo_box.Model.AppendValues (brush.Name, brush);
+			}
+
+			brush_combo_box.ComboBox.Active = 0;
+		}
+
 		#region Mouse Handlers
 		protected override void OnMouseDown (DrawingArea canvas, ButtonPressEventArgs args, PointD point)
 		{
@@ -102,63 +152,55 @@ namespace Pinta.Tools
 			Document doc = PintaCore.Workspace.ActiveDocument;
 
 			if (mouse_button == 1) {
-				StrokeColor = PintaCore.Palette.PrimaryColor;
-				FillColor = PintaCore.Palette.SecondaryColor;
+				stroke_color = PintaCore.Palette.PrimaryColor;
 			} else if (mouse_button == 3) {
-				StrokeColor = PintaCore.Palette.SecondaryColor;
-				FillColor = PintaCore.Palette.PrimaryColor;
+				stroke_color = PintaCore.Palette.SecondaryColor;
 			} else {
-				LastPoint = point_empty;
+				last_point = point_empty;
 				return;
 			}
 
 			// TODO: also multiply by pressure
-			StrokeColor = new Color (StrokeColor.R, StrokeColor.G, StrokeColor.B,
-				StrokeColor.A * active_brush.StrokeAlphaMultiplier);
+			stroke_color = new Color (stroke_color.R, stroke_color.G, stroke_color.B,
+				stroke_color.A * active_brush.StrokeAlphaMultiplier);
 
 			int x = (int)point.X;
 			int y = (int)point.Y;
 
-			if (LastPoint.Equals (point_empty))
-				LastPoint = new Point (x, y);
+			if (last_point.Equals (point_empty))
+				last_point = new Point (x, y);
 
 			if (doc.Workspace.PointInCanvas (point))
 				surface_modified = true;
 
-			var surf = doc.CurrentLayer.Surface;
+			var surf = doc.CurrentUserLayer.Surface;
 			var invalidate_rect = Gdk.Rectangle.Zero;
 			var brush_width = BrushWidth;
 
-			Surface = surf;
+			using (var g = new Context (surf)) {
+				g.AppendPath (doc.Selection.SelectionPath);
+				g.FillRule = FillRule.EvenOdd;
+				g.Clip ();
 
-			using (Drawable = new Context (surf)) {
-				Drawable.AppendPath (doc.Selection.SelectionPath);
-				Drawable.FillRule = FillRule.EvenOdd;
-				Drawable.Clip ();
+				g.Antialias = UseAntialiasing ? Antialias.Subpixel : Antialias.None;
+				g.LineWidth = brush_width;
+				g.LineJoin = LineJoin.Round;
+				g.LineCap = BrushWidth == 1 ? LineCap.Butt : LineCap.Round;
+				g.SetSourceColor (stroke_color);
 
-				Drawable.Antialias = UseAntialiasing ? Antialias.Subpixel : Antialias.None;
-				Drawable.LineWidth = brush_width;
-				Drawable.LineJoin = LineJoin.Round;
-				Drawable.LineCap = BrushWidth == 1 ? LineCap.Butt : LineCap.Round;
-				Drawable.Color = StrokeColor;
-
-				active_brush.Tool = this;
-				invalidate_rect = active_brush.DoMouseMove (x, y, LastPoint.X, LastPoint.Y);
-				active_brush.Tool = null;
+                invalidate_rect = active_brush.DoMouseMove (g, stroke_color, surf,
+				                                            x, y, last_point.X, last_point.Y);
 			}
 
-			Surface = null;
-			Drawable = null;
-
 			// If we draw partially offscreen, Cairo gives us a bogus
 			// dirty rectangle, so redraw everything.
 			if (doc.Workspace.IsPartiallyOffscreen (invalidate_rect)) {
 				doc.Workspace.Invalidate ();
 			} else {
-				doc.Workspace.Invalidate (invalidate_rect);
+				doc.Workspace.Invalidate (doc.ClampToImageSize (invalidate_rect));
 			}
 
-			LastPoint = new Point (x, y);
+			last_point = new Point (x, y);
 		}
 		#endregion
 	}
