@@ -28,20 +28,23 @@ using System;
 using Cairo;
 using Gtk;
 using Pinta.Core;
+using System.Collections.Generic;
+using ClipperLibrary;
 
 namespace Pinta.Tools
 {
-	public abstract class SelectTool : ShapeTool
+	public abstract class SelectTool : SelectShapeTool
 	{
 		private PointD reset_origin;
 		private PointD shape_end;
 		private ToolControl [] controls = new ToolControl [8];
-		protected SelectionHistoryItem hist;
+        private int? active_control;
+		private SelectionHistoryItem hist;
 		public override Gdk.Key ShortcutKey { get { return Gdk.Key.S; } }
 		protected override bool ShowAntialiasingButton { get { return false; } }
-		bool handler_active = false;
 		private Gdk.Cursor cursor_hand;
 		bool is_hand_cursor = false;
+        private CombineMode combine_mode;
 
 		public SelectTool ()
 		{
@@ -49,10 +52,11 @@ namespace Pinta.Tools
 			cursor_hand = new Gdk.Cursor (PintaCore.Chrome.Canvas.Display, PintaCore.Resources.GetIcon ("Tools.Pan.png"), 0, 0);
 		}
 
-		#region ToolBar
+        #region ToolBar
 		// We don't want the ShapeTool's toolbar
 		protected override void BuildToolBar (Toolbar tb)
 		{
+            PintaCore.Workspace.SelectionHandler.BuildToolbar (tb);
 		}
 		#endregion
 		
@@ -63,47 +67,74 @@ namespace Pinta.Tools
 			if (is_drawing)
 				return;
 
-			reset_origin = args.Event.GetPoint ();
+			Document doc = PintaCore.Workspace.ActiveDocument;
+			hist = new SelectionHistoryItem(Icon, Name);
+			hist.TakeSnapshot();
 
-			Document doc = PintaCore.Workspace.ActiveDocument; 
-			if (!handler_active || !HandleResize (point.X, point.Y)) {			
-				double x = Utility.Clamp (point.X, 0, doc.ImageSize.Width - 1);
-				double y = Utility.Clamp (point.Y, 0, doc.ImageSize.Height - 1);
+			reset_origin = args.Event.GetPoint();
 
-				shape_origin = new PointD (x, y);
-				is_drawing = true;
+            active_control = HandleResize (point);
+			if (!active_control.HasValue)
+			{
+				combine_mode = PintaCore.Workspace.SelectionHandler.DetermineCombineMode(args);
+
+				double x = Utility.Clamp(point.X, 0, doc.ImageSize.Width - 1);
+				double y = Utility.Clamp(point.Y, 0, doc.ImageSize.Height - 1);
+				shape_origin = new PointD(x, y);
+
+                doc.PreviousSelection.Dispose ();
+				doc.PreviousSelection = doc.Selection.Clone();
+				doc.Selection.SelectionPolygons.Clear();
+
+                // The bottom right corner should be selected.
+                active_control = 3;
 			}
-			hist = new SelectionHistoryItem (Icon, Name);
-			hist.TakeSnapshot ();
-		}
 
+            is_drawing = true;
+		}
+		
 		protected override void OnMouseUp (DrawingArea canvas, ButtonReleaseEventArgs args, Cairo.PointD point)
 		{
+			Document doc = PintaCore.Workspace.ActiveDocument;
 
 			// If the user didn't move the mouse, they want to deselect
 			int tolerance = 0;
 			if (Math.Abs (reset_origin.X - args.Event.X) <= tolerance && Math.Abs (reset_origin.Y - args.Event.Y) <= tolerance) {
 				PintaCore.Actions.Edit.Deselect.Activate ();
-				hist.Dispose ();
-				hist = null;
-				handler_active = false;
-				Document doc = PintaCore.Workspace.ActiveDocument;
+                if (hist != null)
+                {
+                    hist.Dispose();
+                    hist = null;
+                }
 				doc.ToolLayer.Clear ();
 			} else {
-				if (hist != null)
-					PintaCore.Workspace.ActiveDocument.History.PushNewItem (hist);
+				ReDraw(args.Event.State);
+				if (doc.Selection != null)
+				{
+                    SelectionModeHandler.PerformSelectionMode (combine_mode,
+                        DocumentSelection.ConvertToPolygonSet (doc.Selection.SelectionPolygons));
 
-				handler_active = true;
-				hist = null;
+                    doc.Selection.selOrigin = shape_origin;
+                    doc.Selection.selEnd = shape_end;
+					PintaCore.Workspace.Invalidate();
+				}
+                if (hist != null)
+                {
+                    doc.History.PushNewItem(hist);
+                    hist = null;
+                }
 			}
 
 			is_drawing = false;
+            active_control = null;
+
+            // Update the mouse cursor.
+            CheckHandlerCursor (point);
 		}
 
-		protected override void OnDeactivated ()
+		protected override void OnDeactivated(BaseTool newTool)
 		{
-			base.OnDeactivated ();
-			handler_active = false;
+			base.OnDeactivated (newTool);
 			if (PintaCore.Workspace.HasOpenDocuments) {
 				Document doc = PintaCore.Workspace.ActiveDocument;
 				doc.ToolLayer.Clear ();
@@ -113,7 +144,6 @@ namespace Pinta.Tools
 		protected override void OnCommit ()
 		{
 			base.OnCommit ();
-			handler_active = false;
 			if (PintaCore.Workspace.HasOpenDocuments) {
 				Document doc = PintaCore.Workspace.ActiveDocument;
 				doc.ToolLayer.Clear ();
@@ -122,30 +152,30 @@ namespace Pinta.Tools
 
 		protected override void OnMouseMove (object o, MotionNotifyEventArgs args, Cairo.PointD point)
 		{
-			if (!is_drawing) {
-				CheckHandlerCursor (point.X, point.Y);
-				return;
-			}
 			Document doc = PintaCore.Workspace.ActiveDocument;
 
-			double x = Utility.Clamp (point.X, 0, doc.ImageSize.Width - 1);
-			double y = Utility.Clamp (point.Y, 0, doc.ImageSize.Height - 1);
+			if (!is_drawing)
+			{
+                CheckHandlerCursor (point);
+                return;
+			}
 
-			shape_end = new PointD (x, y);
-			ReDraw (args.Event.State);
+            double x = Utility.Clamp(point.X, 0, doc.ImageSize.Width - 1);
+            double y = Utility.Clamp(point.Y, 0, doc.ImageSize.Height - 1);
+            controls[active_control.Value].HandleMouseMove (x, y, args.Event.State);
+
+            ReDraw(args.Event.State);
+			
+			if (doc.Selection != null)
+			{
+                SelectionModeHandler.PerformSelectionMode (combine_mode,
+                    DocumentSelection.ConvertToPolygonSet (doc.Selection.SelectionPolygons));
+				PintaCore.Workspace.Invalidate();
+			}
 		}
 
-		protected void RefreshHandler (Cairo.Rectangle r)
+		protected void RefreshHandler ()
 		{
-			/*controls[0].Position = new PointD (r.X, r.Y);
-			controls[1].Position = new PointD (r.X, r.GetBottom ());
-			controls[2].Position = new PointD (r.GetRight (), r.Y);
-			controls[3].Position = new PointD (r.GetRight (), r.GetBottom ());
-			controls[4].Position = new PointD (r.X, r.Y + (r.Height / 2));
-			controls[5].Position = n(ew PointD (r.X + (r.Width / 2), r.Y);
-			controls[6].Position = new PointD (r.GetRight (), r.Y + (r.Height / 2));
-			controls[7].Position = new PointD (r.X + (r.Width / 2), r.GetBottom ());
-			*/
 			controls[0].Position = new PointD (shape_origin.X, shape_origin.Y);
 			controls[1].Position = new PointD (shape_origin.X, shape_end.Y);
 			controls[2].Position = new PointD (shape_end.X, shape_origin.Y);
@@ -154,7 +184,6 @@ namespace Pinta.Tools
 			controls[5].Position = new PointD ((shape_origin.X + shape_end.X) / 2, shape_origin.Y);
 			controls[6].Position = new PointD (shape_end.X, (shape_origin.Y + shape_end.Y) / 2);
 			controls[7].Position = new PointD ((shape_origin.X + shape_end.X) / 2, shape_end.Y);
-
 		}
 
 		public void ReDraw (Gdk.ModifierType state)
@@ -178,12 +207,11 @@ namespace Pinta.Tools
 					else
 						shape_end.Y = shape_origin.Y - dx;
 			}
+
 			Cairo.Rectangle rect = Utility.PointsToRectangle (shape_origin, shape_end, constraint);
-			RefreshHandler (rect);
 			Rectangle dirty = DrawShape (rect, doc.SelectionLayer);
 
-			DrawHandler (doc.ToolLayer);
-			doc.Workspace.Invalidate ();
+			updateHandler();
 
 			last_dirty = dirty;
 		}
@@ -272,29 +300,29 @@ namespace Pinta.Tools
 			});
 		}
 
-		public bool HandleResize (double x, double y)
+		public int? HandleResize (PointD point)
 		{
-			foreach (ToolControl ct in controls) {
-				if (ct.Handle (this, new PointD (x, y ))) {
-
-					return true;
-				}
-			}
-			return false;
+            for (int i = 0; i < controls.Length; ++i)
+            {
+                if (controls[i].IsInside (point))
+                    return i;
+            }
 
+			return null;
 		}
 
 		public void DrawHandler (Layer layer)
 		{
 			layer.Clear ();
+			
 			foreach (ToolControl ct in controls)
 				ct.Render (layer);
 		}
 
-		public void CheckHandlerCursor (double x, double y)
+		public void CheckHandlerCursor (PointD point)
 		{
 			foreach (ToolControl ct in controls) {
-				if (ct.IsInside (x, y)) {
+				if (ct.IsInside (point)) {
 					if (!is_hand_cursor) {
 						SetCursor (cursor_hand);
 						is_hand_cursor = true;
@@ -302,6 +330,7 @@ namespace Pinta.Tools
 					return;
 				}
 			}
+
 			if (is_hand_cursor) {
 				SetCursor (DefaultCursor);
 				is_hand_cursor = false;
@@ -309,5 +338,45 @@ namespace Pinta.Tools
 		}
 
 		#endregion
+
+		public override void AfterUndo()
+		{
+			base.AfterUndo();
+
+			Document doc = PintaCore.Workspace.ActiveDocument;
+
+            if (PintaCore.Tools.CurrentTool == this)
+                doc.ToolLayer.Hidden = false;
+
+			shape_origin = doc.Selection.selOrigin;
+			shape_end = doc.Selection.selEnd;
+			updateHandler();
+		}
+
+		public override void AfterRedo()
+		{
+			base.AfterRedo();
+
+			Document doc = PintaCore.Workspace.ActiveDocument;
+
+            if (PintaCore.Tools.CurrentTool == this)
+                doc.ToolLayer.Hidden = false;
+
+			shape_origin = doc.Selection.selOrigin;
+			shape_end = doc.Selection.selEnd;
+			updateHandler();
+		}
+
+		/// <summary>
+		/// Update the selection handler positioning and drawing.
+		/// </summary>
+		private void updateHandler()
+		{
+			Document doc = PintaCore.Workspace.ActiveDocument;
+
+			RefreshHandler();
+			DrawHandler(doc.ToolLayer);
+			PintaCore.Workspace.Invalidate();
+		}
 	}
 }
