@@ -50,15 +50,19 @@ namespace Pinta.Actions
 			Gtk.Clipboard cb = Gtk.Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
 
 			if (cb.WaitIsImageAvailable ()) {
-				Gdk.Pixbuf image = cb.WaitForImage ();
-				Gdk.Size size = new Gdk.Size (image.Width, image.Height);
+				using (Gdk.Pixbuf image = cb.WaitForImage ()) {
+					if (image != null) {
+						Gdk.Size size = new Gdk.Size (image.Width, image.Height);
 
-				PintaCore.Workspace.NewDocument (size, true);
-				PintaCore.Actions.Edit.Paste.Activate ();
-				PintaCore.Actions.Edit.Deselect.Activate ();
-			} else {
-				Pinta.Dialogs.ClipboardEmptyDialog.Show ();
+						PintaCore.Workspace.NewDocument (size, new Cairo.Color (0, 0, 0, 0));
+						PintaCore.Actions.Edit.Paste.Activate ();
+						PintaCore.Actions.Edit.Deselect.Activate ();
+						return;
+					}
+				}
 			}
+
+			Pinta.Core.Document.ShowClipboardEmptyDialog ();
 		}
 	}
 }
