@@ -45,23 +45,34 @@ namespace Pinta.Actions
 
 		private void Activated (object sender, EventArgs e)
 		{
-			NewImageDialog dialog = new NewImageDialog ();
-
-			if (!TryUseClipboardImageSize (dialog))
+			int imgWidth = 0;
+			int imgHeight = 0;
+			var bg_type = NewImageDialog.BackgroundType.White;
+            var using_clipboard = true;
+			
+			// Try to get the dimensions of an image on the clipboard
+			// for the initial width and height values on the NewImageDialog
+			if (!GetClipboardImageSize (out imgWidth, out imgHeight))
 			{
-				dialog.NewImageWidth = PintaCore.Settings.GetSetting<int> ("new-image-width", 800);
-				dialog.NewImageHeight = PintaCore.Settings.GetSetting<int> ("new-image-height", 600);
-			}
+				// An image was not on the clipboard,
+				// so use saved dimensions from settings
+				imgWidth = PintaCore.Settings.GetSetting<int> ("new-image-width", 800);
+				imgHeight = PintaCore.Settings.GetSetting<int> ("new-image-height", 600);
+				bg_type = PintaCore.Settings.GetSetting<NewImageDialog.BackgroundType> (
+					"new-image-bg", NewImageDialog.BackgroundType.White);
+                using_clipboard = false;
+            }
 
-			dialog.WindowPosition = Gtk.WindowPosition.CenterOnParent;
+			var dialog = new NewImageDialog (imgWidth, imgHeight, bg_type, using_clipboard);
 
 			int response = dialog.Run ();
 
 			if (response == (int)Gtk.ResponseType.Ok) {
-				PintaCore.Workspace.NewDocument (new Gdk.Size (dialog.NewImageWidth, dialog.NewImageHeight), false);
+				PintaCore.Workspace.NewDocument (new Gdk.Size (dialog.NewImageWidth, dialog.NewImageHeight), dialog.NewImageBackground);
 
 				PintaCore.Settings.PutSetting ("new-image-width", dialog.NewImageWidth);
 				PintaCore.Settings.PutSetting ("new-image-height", dialog.NewImageHeight);
+				PintaCore.Settings.PutSetting ("new-image-bg", dialog.NewImageBackgroundType);
 				PintaCore.Settings.SaveSettings ();
 			}
 
@@ -69,12 +80,16 @@ namespace Pinta.Actions
 		}
 
 		/// <summary>
-		/// Sets the dialog to use the clipboard image's dimensions, if possible.
+		/// Gets the width and height of an image on the clipboard,
+		/// if available.
 		/// </summary>
-		/// <returns>True if an image was on the clipboard, false otherwise.</returns>
-		private static bool TryUseClipboardImageSize (NewImageDialog dialog)
+		/// <param name="width">Destination for the image width.</param>
+		/// <param name="height">Destination for the image height.</param>
+		/// <returns>True if dimensions were available, false otherwise.</returns>
+		private static bool GetClipboardImageSize (out int width, out int height)
 		{
 			bool clipboardUsed = false;
+			width = height = 0;
 
 			Gtk.Clipboard cb = Gtk.Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
 			if (cb.WaitIsImageAvailable ())
@@ -83,8 +98,8 @@ namespace Pinta.Actions
 				if (image != null)
 				{
 					clipboardUsed = true;
-					dialog.NewImageWidth = image.Width;
-					dialog.NewImageHeight = image.Height;
+					width = image.Width;
+					height = image.Height;
 					image.Dispose ();
 				}
 			}
