@@ -64,7 +64,7 @@ namespace Pinta.Actions
 					return;
 
 				// If the document already has a filename, just re-save it
-				e.Cancel = !SaveFile (e.Document, null, null);
+				e.Cancel = !SaveFile (e.Document, null, null, PintaCore.Chrome.MainWindow);
 			}
 		}
 
@@ -154,7 +154,16 @@ namespace Pinta.Actions
 					format = format_type;
 
 				PintaCore.System.LastDialogDirectory = fcd.CurrentFolder;
-				SaveFile (document, file, format);
+
+				// If saving the file failed or was cancelled, let the user select
+				// a different file type.
+				if (!SaveFile (document, file, format, fcd))
+					continue;
+
+				//The user is saving the Document to a new file, so technically it
+				//hasn't been saved to its associated file in this session.
+				document.HasBeenSavedInSession = false;
+
 				RecentManager.Default.AddFull (fcd.Uri, PintaCore.System.RecentData);
 				PintaCore.System.ImageFormats.SetDefaultFormat (Path.GetExtension (file));
 
@@ -169,7 +178,7 @@ namespace Pinta.Actions
 			return false;
 		}
 
-		private bool SaveFile (Document document, string file, FormatDescriptor format)
+		private bool SaveFile (Document document, string file, FormatDescriptor format, Window parent)
 		{
 			if (string.IsNullOrEmpty (file))
 				file = document.PathAndFileName;
@@ -178,7 +187,7 @@ namespace Pinta.Actions
 				format = PintaCore.System.ImageFormats.GetFormatByFile (file);
 
 			if (format == null || format.IsReadOnly ()) {
-				MessageDialog md = new MessageDialog (PintaCore.Chrome.MainWindow, DialogFlags.Modal, MessageType.Error, ButtonsType.Ok, Catalog.GetString ("Pinta does not support saving images in this file format."), file);
+				MessageDialog md = new MessageDialog (parent, DialogFlags.Modal, MessageType.Error, ButtonsType.Ok, Catalog.GetString ("Pinta does not support saving images in this file format."), file);
 				md.Title = Catalog.GetString ("Error");
 
 				md.Run ();
@@ -189,7 +198,7 @@ namespace Pinta.Actions
 			// If the user tries to save over a read only file, give a more informative error message than "Unhandled Exception"
 			FileInfo file_info = new FileInfo (file);
 			if (file_info.Exists && file_info.IsReadOnly) {
-				MessageDialog md = new MessageDialog (PintaCore.Chrome.MainWindow, DialogFlags.Modal, MessageType.Error,
+				MessageDialog md = new MessageDialog (parent, DialogFlags.Modal, MessageType.Error,
 					ButtonsType.Ok, Catalog.GetString ("Cannot save read only file."));
 				md.Title = Catalog.GetString ("Error");
 
@@ -202,14 +211,14 @@ namespace Pinta.Actions
 			PintaCore.Tools.Commit ();
 
 			try {
-				format.Exporter.Export (document, file);
+				format.Exporter.Export (document, file, parent);
 			} catch (GLib.GException e) { // Errors from GDK
 				if (e.Message == "Image too large to be saved as ICO") {
 					string primary = Catalog.GetString ("Image too large");
 					string secondary = Catalog.GetString ("ICO files can not be larger than 255 x 255 pixels.");
 					string message = string.Format (markup, primary, secondary);
 
-					MessageDialog md = new MessageDialog (PintaCore.Chrome.MainWindow, DialogFlags.Modal, MessageType.Error,
+					MessageDialog md = new MessageDialog (parent, DialogFlags.Modal, MessageType.Error,
 					ButtonsType.Ok, message);
 
 					md.Run ();
@@ -218,11 +227,18 @@ namespace Pinta.Actions
 				} else {
 					throw e; // Only catch exceptions we know the reason for
 				}
+			} catch (OperationCanceledException) {
+				return false;
 			}
 
 			document.Filename = Path.GetFileName (file);
 			document.IsDirty = false;
 
+			PintaCore.Tools.CurrentTool.DoAfterSave();
+
+			//Now the Document has been saved to the file it's associated with in this session.
+			document.HasBeenSavedInSession = true;
+
 			return true;
 		}
 
