@@ -59,14 +59,12 @@ namespace Pinta
 		internal uint TimerHandle;
 
 		string[] authors = new string[] {
-			"Adolfo Jayme Barrientos",
-			"Akshara Proddatoori",
 			"Andrew Davis",
-			"Balló György",
 			"Cameron White",
-			"John Burak",
+			"Don McComb",
+			"Jonathan Bergknoff",
 			"Jonathan Pobst",
-			"Olivier Dufour",
+            "Marius Ungureanu",
 			"Robert Nordan"
 		};
 
@@ -74,15 +72,22 @@ namespace Pinta
 			"Aaron Bockover",
 			"Adam Doppelt",
 			"Adolfo Jayme Barrientos",
+			"Akshara Proddatoori",
+			"Andrew Davis",
 			"Anirudh Sanjeev",
+			"Balló György",
 			"Cameron White",
 			"Ciprian Mustiata",
+			"David Nabraczky",
+			"Don McComb",
+			"Elvis Alistar",
 			"Felix Schmutz",
 			"Greg Lowe",
 			"Hanh Pham",
 			"James Gifford",
 			"Jean-Michel Bea",
 			"Joe Hillenbrand",
+			"John Burak",
 			"Jon Rimmer",
 			"Jonathan Pobst",
 			"Juergen Obernolte",
@@ -97,7 +102,8 @@ namespace Pinta
 			"Obinou Conseil",
 			"Olivier Dufour",
 			"Richard Cohn",
-			"Robert Nordan"
+			"Robert Nordan",
+			"Tom Kadwill"
 		};
 
 		public ScrollBox ()
@@ -250,8 +256,6 @@ namespace Pinta
 			layout.Width = w * (int)Pango.Scale.PangoScale;
 			layout.Wrap = Pango.WrapMode.Word;
 			layout.Alignment = Pango.Alignment.Center;
-			FontDescription fd = FontDescription.FromString ("Tahoma 10");
-			layout.FontDescription = fd;
 			layout.SetMarkup (CreditText);
 
 			backGc = new Gdk.GC (GdkWindow);
