@@ -28,6 +28,7 @@
 using System;
 using Gtk;
 using Mono.Unix;
+using Pinta.Core;
 
 namespace Pinta
 {
@@ -39,7 +40,7 @@ namespace Pinta
 			label.Markup = String.Format (
 			    "<b>{0}</b>\n    {1}",
 			    Catalog.GetString ("Version"),
-			    "1.4");
+                            PintaCore.ApplicationVersion);
 			    
 			HBox hBoxVersion = new HBox ();
 			hBoxVersion.PackStart (label, false, false, 5);
@@ -54,7 +55,7 @@ namespace Pinta
 
 			label = null;
 			label = new Label ();
-			label.Markup = string.Format ("<b>{0}</b>\n    (c) 2010-2012 {1}", Catalog.GetString ("Copyright"), Catalog.GetString ("by Pinta contributors"));
+			label.Markup = string.Format ("<b>{0}</b>\n    (c) 2010-2015 {1}", Catalog.GetString ("Copyright"), Catalog.GetString ("by Pinta contributors"));
 			HBox hBoxCopyright = new HBox ();
 			hBoxCopyright.PackStart (label, false, false, 5);
 			this.PackStart (hBoxCopyright, false, true, 5);
