@@ -31,6 +31,7 @@
 using System;
 using Gtk;
 using Mono.Unix;
+using Pinta.Core;
 
 namespace MonoDevelop.Components.Docking
 {
@@ -269,9 +270,9 @@ namespace MonoDevelop.Components.Docking
 				cr.RelLineTo (0, -rect.Height);
 				cr.ClosePath ();
 				Cairo.SolidPattern solidPattern = new Cairo.SolidPattern (gcol);
-				cr.Pattern = solidPattern;
+				cr.SetSource (solidPattern);
 				cr.FillPreserve ();
-				solidPattern.Destroy ();
+				solidPattern.Dispose ();
 			}
 			
 			header.GdkWindow.DrawRectangle (frame.Style.DarkGC (Gtk.StateType.Normal), false, rect);
@@ -390,14 +391,16 @@ namespace MonoDevelop.Components.Docking
 					cr.RelLineTo (-rect.Width, 0);
 					cr.RelLineTo (0, -rect.Height);
 					cr.ClosePath ();
-					Cairo.Gradient pat = new Cairo.LinearGradient (rect.X, rect.Y, rect.X, rect.Bottom);
-					Cairo.Color color1 = gcol;
-					pat.AddColorStop (0, color1);
-					gcol.L -= 0.1;
-					if (gcol.L < 0) gcol.L = 0;
-					pat.AddColorStop (1, gcol);
-					cr.Pattern = pat;
-					cr.FillPreserve ();
+					using (var pat = new Cairo.LinearGradient (rect.X, rect.Y, rect.X, rect.Bottom)) {
+						Cairo.Color color1 = gcol;
+						pat.AddColorStop (0, color1);
+						gcol.L -= 0.1;
+						if (gcol.L < 0)
+							gcol.L = 0;
+						pat.AddColorStop (1, gcol);
+						cr.SetSource (pat);
+						cr.FillPreserve ();
+					}
 				}
 			}
 			
