@@ -30,6 +30,7 @@ using Mono.Addins;
 using Mono.Unix;
 using MonoDevelop.Components.Docking;
 using Pinta.Core;
+using Pinta.MacInterop;
 
 namespace Pinta
 {
@@ -39,9 +40,9 @@ namespace Pinta
 		ScrolledWindow sw;
 		DockFrame dock;
 		Menu show_pad;
-		
-		ActionHandlers dialog_handlers;
-		
+
+		CanvasPad canvas_pad;
+
 		public MainWindow ()
 		{
 			// Build our window
@@ -49,14 +50,30 @@ namespace Pinta
 
 			// Initialize interface things
 			window_shell.AddAccelGroup (PintaCore.Actions.AccelGroup);
-			dialog_handlers = new ActionHandlers ();
+			new ActionHandlers ();
 
 			PintaCore.Chrome.InitializeProgessDialog (new ProgressDialog ());
+			PintaCore.Chrome.InitializeErrorDialogHandler ((parent, message, details) => {
+				System.Console.Error.WriteLine ("Pinta: {0}", details);
+				ErrorDialog errorDialog = new ErrorDialog (parent);				
+				try {
+					errorDialog.SetMessage (message);
+					errorDialog.AddDetails (details);
+					errorDialog.Run ();
+				} finally {
+					errorDialog.Destroy ();
+				}
+			}
+			);
+
 			PintaCore.Initialize ();
 
 			// Initialize extensions
 			AddinManager.Initialize ();
 			AddinManager.Registry.Update ();
+			AddinSetupService setupService = new AddinSetupService (AddinManager.Registry);
+			if (!setupService.AreRepositoriesRegistered ())
+				setupService.RegisterRepositories (true);
 
 			//Look out for any changes in extensions
 			AddinManager.AddExtensionNodeHandler (typeof (IExtension), OnExtensionChanged);
@@ -79,12 +96,74 @@ namespace Pinta
 			window_shell.DeleteEvent += MainWindow_DeleteEvent;
 			window_shell.DragDataReceived += MainWindow_DragDataReceived;
 
+			window_shell.KeyPressEvent += MainWindow_KeyPressEvent;
+			window_shell.KeyReleaseEvent += MainWindow_KeyReleaseEvent;
+
 			// TODO: These need to be [re]moved when we redo zoom support
 			PintaCore.Actions.View.ZoomToWindow.Activated += new EventHandler (ZoomToWindow_Activated);
 			PintaCore.Actions.View.ZoomToSelection.Activated += new EventHandler (ZoomToSelection_Activated);
 			PintaCore.Workspace.ActiveDocumentChanged += ActiveDocumentChanged;
 		}
 
+		[GLib.ConnectBefore]
+		private void MainWindow_KeyPressEvent (object o, KeyPressEventArgs e)
+		{
+            // Give the widget that has focus a first shot at handling the event.
+            // Otherwise, key presses may be intercepted by shortcuts for menu items.
+            if (SendToFocusWidget (e, e.Event))
+                return;
+
+			// Give the Canvas (and by extension the tools)
+			// first shot at handling the event if
+			// the mouse pointer is on the canvas
+			if (IsMouseOnCanvas() || canvas_pad.Canvas.HasFocus)
+			{
+				canvas_pad.Canvas.DoKeyPressEvent (o, e);
+			}
+		}
+
+		[GLib.ConnectBefore]
+		private void MainWindow_KeyReleaseEvent (object o, KeyReleaseEventArgs e)
+		{
+            if (SendToFocusWidget (e, e.Event))
+                return;
+
+			// Give the Canvas (and by extension the tools)
+			// first shot at handling the event if
+			// the mouse pointer is on the canvas
+			if (IsMouseOnCanvas() || canvas_pad.Canvas.HasFocus)
+			{
+				canvas_pad.Canvas.DoKeyReleaseEvent (o, e);
+			}
+		}
+
+        private bool SendToFocusWidget (GLib.SignalArgs args, Gdk.EventKey e)
+        {
+            var widget = window_shell.Focus;
+            if (widget != null && widget.ProcessEvent (e))
+            {
+                args.RetVal = true;
+                return true;
+            }
+
+            return false;
+        }
+
+		// Check if the mouse pointer is on the canvas
+		private bool IsMouseOnCanvas()
+		{
+			int x = 0;
+			int y = 0;
+
+			// Get the position of the mouse pointer relative
+			// to canvas scrolled window top-left corner
+			sw.GetPointer (out x, out y);
+
+			// Check if the pointer is on the canvas
+			return (x > 0) && (x < sw.Allocation.Width) &&
+			    (y > 0) && (y < sw.Allocation.Height);
+		}
+
 		// Called when an extension node is added or removed
 		private void OnExtensionChanged (object s, ExtensionNodeEventArgs args)
 		{
@@ -130,6 +209,7 @@ namespace Pinta
 			main_menu.Append (new Gtk.Action ("layers", Catalog.GetString ("_Layers")).CreateMenuItem ());
 			main_menu.Append (new Gtk.Action ("adjustments", Catalog.GetString ("_Adjustments")).CreateMenuItem ());
 			main_menu.Append (new Gtk.Action ("effects", Catalog.GetString ("Effe_cts")).CreateMenuItem ());
+			main_menu.Append (new Gtk.Action ("addins", Catalog.GetString ("A_dd-ins")).CreateMenuItem ());
 
 			MenuItem window_menu = (MenuItem)new Gtk.Action ("window", Catalog.GetString ("_Window")).CreateMenuItem ();
 			window_menu.Submenu = new Menu ();
@@ -225,7 +305,7 @@ namespace Pinta
 			palettepad.Initialize (dock, show_pad);
 
 			// Canvas pad
-			var canvas_pad = new CanvasPad ();
+			canvas_pad = new CanvasPad ();
 			canvas_pad.Initialize (dock, show_pad);
 
 			sw = canvas_pad.ScrolledWindow;
@@ -379,7 +459,7 @@ namespace Pinta
 
 				PintaCore.Workspace.Scale = ratio;
 				PintaCore.Actions.View.SuspendZoomUpdate ();
-				(PintaCore.Actions.View.ZoomComboBox.ComboBox as ComboBoxEntry).Entry.Text = string.Format ("{0}%", (int)(PintaCore.Workspace.Scale * 100));
+				(PintaCore.Actions.View.ZoomComboBox.ComboBox as ComboBoxEntry).Entry.Text = ViewActions.ToPercent (PintaCore.Workspace.Scale);
 				PintaCore.Actions.View.ResumeZoomUpdate ();
 			}
 
@@ -389,10 +469,8 @@ namespace Pinta
 		private void ActiveDocumentChanged (object sender, EventArgs e)
 		{
 			if (PintaCore.Workspace.HasOpenDocuments) {
-				int zoom = (int)(PintaCore.Workspace.ActiveWorkspace.Scale * 100);
-			
 				PintaCore.Actions.View.SuspendZoomUpdate ();
-				(PintaCore.Actions.View.ZoomComboBox.ComboBox as Gtk.ComboBoxEntry).Entry.Text = string.Format ("{0}%", zoom);
+				(PintaCore.Actions.View.ZoomComboBox.ComboBox as Gtk.ComboBoxEntry).Entry.Text = ViewActions.ToPercent (PintaCore.Workspace.Scale);
 				PintaCore.Actions.View.ResumeZoomUpdate ();
 
 				PintaCore.Workspace.OnCanvasSizeChanged ();
